#!/bin/sh
. ./install.cf

fix_old_param()
{
    local fini="/etc/failover.ini"
    local value
    local rb

    if [ -f "$fini" ]; then
        rb=`utils/read_par -f $fini -s misc -p reboot`

        # check if already set up
        if [ -z "$rb" ]; then

            # Convert 'usewatchdog' parameter to 'reboot' parameter in failover.ini
            value=`utils/read_par -f "$fini" -s watchdog -p usewatchdog`
            if [ "$?" -ne 0 ];  then
                value="no"
            fi

            utils/write_par -f $fini -s misc -p reboot -v $value
        fi

        # delete old sections
        utils/del_sec -f $fini -s iplir
        utils/del_sec -f $fini -s mftp
        utils/del_sec -f $fini -s watchdog
    fi
}

install_sample()
{
    local type="$1"
    local target="$2"

    if [ "$type" = "INSTALL" ]; then
        if [ -f scripts/$target ]; then
            $INSTALL_CONF scripts/$target $ETCPATH
            log_vipnet_file $ETCPATH/$target
            _log_message "Install $target to $ETCPATH"
        fi
    else
        if [ -f scripts/$target -a ! -r $ETCPATH/$target ]; then
            $INSTALL_CONF scripts/$target $ETCPATH
            log_vipnet_file $ETCPATH/$target
            _log_message "Install $target to $ETCPATH"
        fi
    fi
}


export LANG=C
export LC_ALL=C
umask 077
trap '_error_exit "Installation ABORTED by signal." 100' HUP INT QUIT TERM PIPE
_init_log

_message "STEP 0: Checking system requirements"
_check_system_requirements

_message "STEP 1: Select ViPNet installation distribution"
# find and select distribution archive
INSTALL_DIR=`pwd`
DISTRIBUTION_DIR=`pwd`
while true
do
  echo "Searching in '$DISTRIBUTION_DIR'..."
  get_files_list $DISTRIBUTION_MASK "DISTRIBUTE"
  get_choice $? "CHANGE"
  DISTRIBUTION_FILE=`files_list_get_item $?`
  unset_files_list

  if [ "$DISTRIBUTION_FILE" = "Change directory" ] ; then
    while true ; do
      echo -n "Enter full path to ViPNet distribution directory:"
      read DISTRIBUTION_DIR
      if [ ! -d "$DISTRIBUTION_DIR" -o -z "$DISTRIBUTION_DIR" ] ; then
        echo "Directory '$DISTRIBUTION_DIR' doesn't exist!"
        is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
      else
        cd "$DISTRIBUTION_DIR" || \
          _error_exit "Unable to change current directory to '$DISTRIBUTION_DIR'! Installation ABORTED." 1
          DISTRIBUTION_DIR=`pwd`
        break
      fi
    done
  else
    break
  fi
done

DISTRIBUTION_PATH="$DISTRIBUTION_DIR/$DISTRIBUTION_FILE"
_log_message "Selected distribution file: '$DISTRIBUTION_PATH'"

# show warning message
if [ -f "$INSTALL_DIR/$WARNING_FILE" ] ; then
  echo "Press ENTER to read warning message"
  read WARDUMMY
  more "$INSTALL_DIR/$WARNING_FILE"
  is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
fi

# create temporary directory
[ ! -d $TMP_DIR ] && \
  _error_exit "Unable to find temporary directory '$TMP_DIR'! Installation ABORTED." 2

TMP_DISTRIBUTION_DIR=$TMP_DIR/dist`date +%s`
if [ ! -d $TMP_DISTRIBUTION_DIR ] ; then
  mkdir $TMP_DISTRIBUTION_DIR || \
    _error_exit "Unable to create '$TMP_DISTRIBUTION_DIR'! Installation ABORTED." 3
fi

_message "STEP 2: Unpacking ViPNet distribution archive"
# unpack distribution archive to temporary directory
cd $TMP_DISTRIBUTION_DIR || \
  _error_exit "Unable to change current directory to '$TMP_DISTRIBUTION_DIR'! Installation ABORTED." 5

( gunzip -c "$DISTRIBUTION_PATH" | tar xf - ) || \
  _error_exit "Unable to unpack '$DISTRIBUTION_PATH'! Installation ABORTED." 6
_message "Unpack '$DISTRIBUTION_PATH' successfull"

# show license message
if [ -f "$INSTALL_DIR/$LICENSE_FILE" ] ; then
  echo "Press ENTER to read License Agreement"
  read LICDUMMY
  more "$INSTALL_DIR/$LICENSE_FILE"
  is_continue "Do you agree to this License (y/n)" || _exit_install 0
fi

_message "STEP 3: Searching for current ViPNet configuration"
# find current ViPNet configuration
install_type="INSTALL"
vipnet_installed="NOTINSTALLED"
if [ -f $PSWFILE ] ; then
  VIPNET_DIR=`cat $PSWFILE | head -n 1`
  if [ -d "$VIPNET_DIR"  ] ; then
    if ! is_empty_dir "$VIPNET_DIR" ; then
      vipnet_installed="INSTALLED"
    else
      VIPNET_DIR=""
    fi
  else
    VIPNET_DIR=""
  fi
fi

if [ "$vipnet_installed" = "INSTALLED" ] ; then
  install_type="UPGRADE"
  _message "Existing ViPNet configuration has been found in '$VIPNET_DIR'."
  is_continue "Do you want to use existing configuration? (y/n)" || \
    install_type="INSTALL"
else
  _message "No current ViPNet configuration found"
fi
_log_message "Selected installation type: $install_type"

_message "STEP 4: Preparing for installation"
# check for running vipnet services
if [ "$vipnet_installed" = "INSTALLED" ] ; then
  _log_message "Checking for running ViPNet services..."
  # check if all ViPNet services are stoped
  if [ -f $NUMBERS_FILE ] ; then
    . $NUMBERS_FILE
  else
     _error_exit "Unable to find '$NUMBERS_FILE' for previous ViPNet installation! Installation ABORTED." 8
  fi

  if is_vipnet_run ; then
    _message "Some ViPNet services are running now and need to be stopped before installation continue!"
    is_continue "Do you want to stop them automatically? (y/n)" || \
      _exit_install 0

    _log_message "Stop running ViPNet services..."
    vipnet_stop
    if is_vipnet_run ; then
      _message "Some ViPNet services are still running now!"
      _error_exit "Please stop them manually and run installation again. Installation ABORTED." 9
    else
      _log_message "All ViPNet services are stoped"
    fi
  else
    _log_message "No running ViPNet services found"
  fi
  if [ "$install_type" = "UPGRADE" ] ; then
    _log_message "Removing old version services sessions:"
    _log_message "$VIPNET_DIR/*.session"
    rm -f $VIPNET_DIR/*.session
  fi
fi

_log_message "Removing services sessions:"
SESSION_FILES="/var/run/*.session /var/run/watch_app_list"
for i in $SESSION_FILES; do
    _log_message "$i"
    rm -f $i
done

if [ "$vipnet_installed" = "INSTALLED" -a "$install_type" = "UPGRADE" ] ; then
# check for additional free space for packet databases new default size (50Mb)
     _update_config;
# correct mftp.conf wrong parameters values
     _check_mftp_conf
fi

_log_message "Copy rules.xsd file for convertor "
mkdir -p $LIBPATH/rules_xsd
cp -f $TMP_DISTRIBUTION_DIR/scripts/rules.xsd $LIBPATH/rules_xsd

get_new_version
if [ "$install_type" = "UPGRADE" ] ; then
    _message "STEP 5: Saving configuration files for previous ViPNet installation"
    save_previous_config
    if [ $? -eq 1 ] ; then
        if is_need_restore ; then
            _message "Updating existing version."
            _message "Your current configuration files are not altered and may become invalid"
            _message "You may have to edit them manually"
            new_vers="$IPLIR_VERS_MAJOR_NEW.$IPLIR_VERS_MINOR_NEW.$IPLIR_VERS_SUBMINOR_NEW"
            old_vers="$IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD.$IPLIR_VERS_SUBMINOR_OLD"
            _log_message "Installation version $new_wers atop newer version $old_vers is made"

            find_config
            res=$?
            echo "Choose one of the following options:"
            echo
            echo "1 - Interrupt installation;"
            echo "2 - Leave current configuration files;"
            if [ $res -eq 0 ] ; then
                echo "3 - To restore a configuration from a database;"
            fi
            read answer

            case "$answer" in
                "1")
                    _exit_install 0
                    ;;
                "2")
                    _message "Curent configuration files will be left uncahnged; You may need additional manual editing"
                    _log_message "Configuration files for version $old_vers will be left"
                    ;;
                "3")
                    _message "The configuration will be restored"
                    _log_message "Restoring configuration from DB "
                    restore_config "$INSTALL_DIR"
                    ;;
                * )
                    _message "Error on input"
                    _exit_install 0
                    ;;
            esac
        fi
    fi

    STORAGE_DB=$VIPNET_DIR/user/storage.db
    if is_need_db_convert ; then
        _message "Converting configurations to new DB format..."

        if [ -f $STORAGE_DB ] ; then
            _log_message "'$STORAGE_DB' is found. It will be removed"
            rm -f $STORAGE_DB
        fi

        #     ,        ,
        #        shell --   :
        export TMP_DISTRIBUTION_DIR LOG_DIR INSTALL_LOG
        ret=`( ( $TMP_DISTRIBUTION_DIR/application/iplir-config-converter --silent  2>&1 ; echo $? >&5 ) >> $LOG_DIR/$INSTALL_LOG ) 5>&1`

        if [ $ret -eq 0 ] ; then
            _message "All configurations are successfully converted and saved in new DB"
        else
            _message "Error occured during convertation process. For details see \"$LOG_DIR/$INSTALL_LOG\""
        fi
    fi

    check_vers_3 "$VIPNET_DIR"
    convert_key=$?

    # Converting firewall rules from 3.6/3.7 to 4.0 format
    if [ $IPLIR_VERS_MAJOR_OLD -eq 3 -a \( $IPLIR_VERS_MINOR_OLD -eq 6 -o $IPLIR_VERS_MINOR_OLD -eq 7 \) ] ; then
        old_vers="$IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD.$IPLIR_VERS_SUBMINOR_OLD"
        OLD_CONF_DIR="$VIPNET_DIR/user/config.$old_vers"
        mkdir -p $OLD_CONF_DIR
        if [ -f $VIPNET_DIR/user/firewall.conf ]; then
            _log_message "Moving $VIPNET_DIR/user/firewall.conf in $OLD_CONF_DIR"
            mv -f $VIPNET_DIR/user/firewall.conf $OLD_CONF_DIR/firewall.conf
        fi
        if [ -f $VIPNET_DIR/user/iplir.conf ]; then
            _log_message "Backuping $VIPNET_DIR/user/iplir.conf to $OLD_CONF_DIR"
            cp -f $VIPNET_DIR/user/iplir.conf $OLD_CONF_DIR/iplir.conf
        fi
        if [ -f $VIPNET_DIR/user/iplir.serv ]; then
            _log_message "Moving $VIPNET_DIR/user/iplir.serv to $OLD_CONF_DIR"
            mv -f $VIPNET_DIR/user/iplir.serv $OLD_CONF_DIR/iplir.serv
        fi
        if [ -f $VIPNET_DIR/user/mftp.conf ]; then
            _log_message "Backuping $VIPNET_DIR/user/mftp.conf to $OLD_CONF_DIR"
            cp -f $VIPNET_DIR/user/mftp.conf $OLD_CONF_DIR/mftp.conf
        fi
        if [ -f $VIPNET_DIR/user/events.db ]; then
            _log_message "Moving $VIPNET_DIR/user/events.db to $OLD_CONF_DIR"
            mv -f $VIPNET_DIR/user/events.db $OLD_CONF_DIR/events.db
        fi
        cp -f $VIPNET_DIR/user/iplir.conf-* $OLD_CONF_DIR
        _message "Converting firewall config rules to new format..."
        if [ -f $VIPNET_DIR/user/user.xml ]; then
            _message "Backuping $VIPNET_DIR/user/user.xml to $VIPNET_DIR/user/user.xml.old"
            _log_message "Backuping $VIPNET_DIR/user/user.xml to $VIPNET_DIR/user/user.xml.old"
            mv -f $VIPNET_DIR/user/user.xml $VIPNET_DIR/user/user.xml.old
            mv -f $VIPNET_DIR/user/user.crg $VIPNET_DIR/user/user.crg.old
        fi
        _log_message "Converting $OLD_CONF_DIR/iplir.conf"
        _log_message "Converting $OLD_CONF_DIR/firewall.conf"
        $TMP_DISTRIBUTION_DIR/application/firewall-config-converter --config $OLD_CONF_DIR \
            --xml $VIPNET_DIR/user/user.xml \
            --crg $VIPNET_DIR/user/user.crg \
            --objects-xml $VIPNET_DIR/user/objects.xml \
            --objects-crg $VIPNET_DIR/user/objects.crg \
               >> $LOG_DIR/$INSTALL_LOG
        ret_fw_converter=$?
        if [ $ret_fw_converter -eq 0 ] ; then
            _message "Firewall rules have been successfully converted and stored into $VIPNET_DIR/user/user.xml"
        else
            _message "Unable to convert firewall rules from previous configuration, see details in \"$LOG_DIR/$INSTALL_LOG\""
            if [ $ret_fw_converter -eq 1 ]; then
                _message "Please convert firewall rules manually"
                _log_message "Please convert firewall rules manually"
            elif [ $ret_fw_converter -eq 2 ]; then
                _message "Please check $OLD_CONF_DIR/iplir.conf for unconverted rules"
                _log_message "Please check $OLD_CONF_DIR/iplir.conf for unconverted rules"
            elif [ $ret_fw_converter -eq 4 ]; then
                _message "Please check $OLD_CONF_DIR/firewall.conf for unconverted rules"
                _log_message "Please check $OLD_CONF_DIR/firewall.conf for unconverted rules"
            elif [ $ret_fw_converter -eq 6 ]; then
                _message "Please check $OLD_CONF_DIR/iplir.conf and $OLD_CONF_DIR/firewall.conf for unconverted rules"
                _log_message "Please check $OLD_CONF_DIR/iplir.conf and $OLD_CONF_DIR/firewall.conf for unconverted rules"
            fi
            is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
        fi
    else
        if [ $IPLIR_VERS_MAJOR_OLD -lt 3 -o \( $IPLIR_VERS_MAJOR_OLD -eq 3 -a $IPLIR_VERS_MINOR_OLD -lt 6 \) ] ; then
            _message "Unable to convert firewall rules because of too old version: $IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD"
            _message "Please convert it manually with command \"vipnet firewall\", see documentation for details."
            _log_message "Unable to convert firewall rules because of too old version: $IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD"
        fi
    fi

    # Convert mftp config file from 3.6/3.7 format
   # if [ $IPLIR_VERS_MAJOR_OLD -eq 3 -a \( $IPLIR_VERS_MINOR_OLD -eq 6 -o $IPLIR_VERS_MINOR_OLD -eq 7 \) -o $IPLIR_VERS_MAJOR_OLD -eq 4 -a $IPLIR_VERS_MINOR_OLD -eq 0 ]; then
   #     old_vers="$IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD.$IPLIR_VERS_SUBMINOR_OLD"
   #     OLD_CONF_DIR="$VIPNET_DIR/user/config.$old_vers"
   #     mkdir -p "$OLD_CONF_DIR"
   #     OLD_MFTP_CONF="$VIPNET_DIR/user/mftp.conf"
   #     NEW_MFTP_CONF="$VIPNET_DIR/user/mftpconf.xml"
   #     if [ -f "$OLD_MFTP_CONF" ]; then
   #         _log_message "Backup $OLD_MFTP_CONF to $OLD_CONF_DIR"
   #         cp -f "$OLD_MFTP_CONF" "$OLD_CONF_DIR/mftp.conf"
   #     fi
   #     _message "Converting mftp config file to new format..."
   #     if [ -f "$NEW_MFTP_CONF" ]; then
   #         _message "Backup $NEW_MFTP_CONF to $NEW_MFTP_CONF.old"
    #        cp -f "$NEW_MFTP_CONF" "$NEW_MFTP_CONF.old"
    #        cp -f "$VIPNET_DIR/user/mftpd-config.crg" "$VIPNET_DIR/user/mftpd-config.crg.old"
    #    fi
     #   _log_message "Converting $OLD_MFTP_CONF"
       # "$TMP_DISTRIBUTION_DIR"/application/mftp-config-converter "$VIPNET_DIR" \
     #          >> "$LOG_DIR/$INSTALL_LOG"
    #    ret_mftp_converter=$?
    #    if [ $ret_mftp_converter -eq 0 ] ; then
    #        _message "Mftp config file has been successfully converted and stored into $NEW_MFTP_CONF"
    #    else
    #        _message "Unable to convert mftp config file from previous configuration, see details in \"$LOG_DIR/$INSTALL_LOG\""
    #        if [ $ret_mftp_converter -eq 1 ]; then
    #            _message "Please convert mftp config file manually"
    #        fi
    #        is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
    #    fi
    #else
    #    if [ $IPLIR_VERS_MAJOR_OLD -lt 3 -o \( $IPLIR_VERS_MAJOR_OLD -eq 3 -a $IPLIR_VERS_MINOR_OLD -lt 6 \) ] ; then
     #       _message "Unable to convert mftp config file because of too old version: $IPLIR_VERS_MAJOR_OLD.$IPLIR_VERS_MINOR_OLD"
     #       _message "Please convert it manually, see documentation for details."
    #    fi
    #fi
fi

_log_message "Checking packet filtering method..."
# check packet filtering method
_check_packet_filtering_method

_log_message "Checking IPv6 support..."
# update $DRIVERS as well
_check_ipv6_support 

_prepare_drivers_build

# show version message
_message "This script will install"
_message `cat "$INSTALL_DIR/version" |head -n 2`
is_continue "Do you want to continue installation? (y/n)" || _exit_install 0

# put ViPNet loading before network services starting
if [ -x $RCPATH/rc.sysvinit -a -x $RCPATH/rc.M ] ; then
  move_vipnet_start $RCPATH/rc.M
fi

_message "STEP 6: Installing drivers"
file_version=version.h
usual_path_version=/lib/modules/`uname -r`/build/include/linux
generated_path_version=/lib/modules/`uname -r`/build/include/generated/uapi/linux
if [ ! -f "$usual_path_version/$file_version" ]; then
  if [ -f "$generated_path_version/$file_version" ]; then
    _log_message "Absent $usual_path_version/$file_version - make link to $generated_path_version/$file_version"
    ln -s "$generated_path_version/$file_version" "$usual_path_version/$file_version"
  else
    _message "Absent $usual_path_version/$file_version and can't find the link: $generated_path_version/$file_version"
    _error_exit "Installation ABORTED." 23
  fi
fi

_log_message "Compiling drivers..."
# compile drivers
cd $TMP_DISTRIBUTION_DIR/src
if make all DRVTRACE=$DRVTRACE >>$LOG_DIR/$INSTALL_LOG 2>&1; then
  cd ..
  _log_message "Drivers compiled"
else
  _message "Error ocured during drivers compiling. See $LOG_DIR/$INSTALL_LOG for details."
  _error_exit "Installation ABORTED." 16
fi

# install drivers
mkdir -p $MODPATH
for drv in $DRIVERS ; do
  drv_name=$drv.ko
  echo "Install $drv_name"
  if [ -f driver/$drv_name ] ; then
    $INSTALL_DRV driver/$drv_name $MODPATH >>$LOG_DIR/$INSTALL_LOG 2>&1 ;
    $DEPMOD -e $MODPATH/$drv_name >>$LOG_DIR/$INSTALL_LOG 2>&1 ;
    log_vipnet_file $MODPATH/$drv_name
    _log_message "Install $drv_name to $MODPATH"
  elif [ -f src/$drv_name ] ; then
    $INSTALL_DRV src/$drv_name $MODPATH >>$LOG_DIR/$INSTALL_LOG 2>&1 ;
    $DEPMOD -e $MODPATH/$drv_name >>$LOG_DIR/$INSTALL_LOG 2>&1 ;
    log_vipnet_file $MODPATH/$drv_name
    _log_message "Install $drv_name to $MODPATH"
  else
    _message "Compiled driver $drv_name was not found to install."
    _log_message "Failed to install $drv_name"
    _error_exit "Installation ABORTED." 24
  fi
done
$DEPMOD 2>/dev/null

_message "STEP 7: Installing applications"
# install libraries
if [ -d lib ] ; then
  _log_message "Installing libraries..."
  rm -rf $LIBPATH
  mkdir -p $LIBPATH
  mkdir -p $CLIPLUGINPATH
  chmod a+rx $LIBPATH $CLIPLUGINPATH
  _log_message "Creating directory $LIBPATH"
  for target in `ls lib/` ; do

    #do some work only for regular files
    if [ -f lib/$target ]
    then
        $INSTALL_LIB lib/$target $LIBPATH
        log_vipnet_file $LIBPATH/$target
        _log_message "Installing $target to $LIBPATH"
    fi
  done
  
  for target in `ls lib/cliplugins` ; do

    #do some work for cliplugins
    if [ -f lib/cliplugins/$target ]
    then
        $INSTALL_LIB lib/cliplugins/$target $CLIPLUGINPATH
        log_vipnet_file $CLIPLUGINPATH/$target
        _log_message "Installing $target to $CLIPLUGINPATH"
    fi
  done

  #in case if we have tls library with NPTL libraries
  if [ -d lib/tls ] ; then
    _log_message "found lib/tls folder"
    _log_message "KERNREL:$KERNREL"
    _log_message "KERNSUBLEVEL:$KERNSUBLEVEL"
    _check_config_futex
    if [ $? -ge "1" ] ; then
      _log_message "futex support is present or .config file was not found"
      #in this case replace NPTL libraryes
      for tlstarget in `ls lib/tls/` ; do
        $INSTALL_LIB lib/tls/$tlstarget $LIBPATH
        log_vipnet_file $LIBPATH/$tlstarget
        _log_message "Install $tlstarget to $LIBPATH"
      done
    else
      _log_message "futex support is NOT present"
    fi
  fi
  # adopt for SELinux (RHEL5 especially)
  if which getenforce >/dev/null 2>&1 &&
      getenforce | grep -qi enforc ; then
    _log_message "SELinux in enforcing mode is detected ..."
    # allow to perform prot change to PROT_EXEC after relocation of
    # shared libraries since vipnet/loader may be not aware of SELinux
    ls $LIBPATH/*.so* | grep -v [.]crg | \
	xargs -i chcon -t textrel_shlib_t {}
  fi
fi
#rewrite NTPL libraries

#copy WebGUI files
if [ -d $TMP_DISTRIBUTION_DIR/lib/services ] ; then
  cp -fR $TMP_DISTRIBUTION_DIR/lib/services $LIBPATH/
  cp -f $TMP_DISTRIBUTION_DIR/scripts/vipnet-web-gui /sbin/vipnet-web-gui
  cp -f $TMP_DISTRIBUTION_DIR/scripts/vipnet-web-gui.crg /sbin/vipnet-web-gui.crg
  chmod a+x /sbin/vipnet-web-gui
  mkdir -p /usr/share/vipnet/webgui
  chmod a+rx /usr/share/vipnet
  cp -fR $TMP_DISTRIBUTION_DIR/resource/ui/* /usr/share/vipnet/webgui
  mkdir -p /usr/share/vipnet/webgui/logs
  ln -snf /var/log/everything.log /usr/share/vipnet/webgui/logs/everything.log
  chmod a+r /usr/share/vipnet/webgui/logs/everything.log
  chmod a+rx -R /usr/share/vipnet/webgui
fi

_log_message "Installing applications ..."

find application -maxdepth 1 -type f '!' -name '*.crg' | while read target; do
  $INSTALL_APP $target $SBINPATH
  log_vipnet_file $SBINPATH/$target
  _log_message "Installing $target to $SBINPATH"
  $INSTALL_CONF $target.crg $SBINPATH
  log_vipnet_file $SBINPATH/$target.crg
  _log_message "Installing $target.crg to $SBINPATH"
done
find contrib -maxdepth 1 -type f '!' -name '*.crg' | while read target; do
  $INSTALL_APP $target $USRBINPATH
  log_vipnet_file $USRBINPATH/$target
  _log_message "Installing $target to $USRBINPATH"
  $INSTALL_CONF $target.crg $USRBINPATH
  log_vipnet_file $USRBINPATH/$target.crg
  _log_message "Installing $target.crg to $USRBINPATH"
done
if [ -f $SBINPATH/watchdogcfg ]; then
  rm -f $SBINPATH/watchdogcfg
  rm -f $SBINPATH/watchdogcfg.crg
  _log_message "Removing watchdogcfg from $SBINPATH"
fi
if [ -f $SBINPATH/watchdog ]; then
  rm -f $SBINPATH/watchdog
  _log_message "Removing watchdog from $SBINPATH"
fi
for target in $SCRIPT ; do
  $INSTALL_SCR scripts/$target $SBINPATH
  log_vipnet_file $SBINPATH/$target
  _log_message "Installing $target to $SBINPATH"
  $INSTALL_CONF scripts/$target.crg $SBINPATH
  log_vipnet_file $SBINPATH/$target.crg
  _log_message "Installing $target.crg to $SBINPATH"
done
for target in "$SCRIPLIR" "$SCRWD" "$SCRMOD" "$SCRFO" "$SCRMFTP" "rmiplir" ; do
    rm -f $RCPATH/init.d/$target
done
for target in $SCRMOD $SCRFO $SCRIPLIR $SCRMFTP; do
  ln -f -s $SBINPATH/$target $RCPATH/init.d/$target
  log_vipnet_file $RCPATH/init.d/$target
  _log_message "Creating symlink $RCPATH/init.d/$target -> $SBINPATH/$target"
done
for target in $CONF ; do
  $INSTALL_CONF scripts/$target $ETCPATH
  log_vipnet_file $ETCPATH/$target
  _log_message "Installing $target to $ETCPATH"
done

 _log_message "Installing vipnet to $ETCPATH/logrotate.d"
$INSTALL_CONF scripts/vipnet.log.conf $ETCPATH/logrotate.d
log_vipnet_file $ETCPATH/logrotate.d/vipnet

for target in $CONFCRG ; do
  $INSTALL_CONF scripts/$target.crg $ETCPATH
  log_vipnet_file $ETCPATH/$target.crg
  _log_message "Installing $target.crg to $ETCPATH"
done

for target in $SAMPLE ; do
    install_sample $install_type $target
done

# configure rc.d scripts
_log_message "Configuring startup scripts ..."
if [ -x /sbin/insserv -a ! -f /etc/init.d/.legacy-bootordering ] ; then
  _log_message "SuSe Linux boot concept is detected"
  INITD_WAY="insserv"
else
  _log_message "System V boot concept is used by default"
  INITD_WAY="sysv"
fi
if [ "${INITD_WAY}" = "insserv" ] ; then
  # support Linux SuSe boot concept
  if [ -f /etc/insserv.conf ] ; then
    for target in "$SCRFO" "$SCRMFTP" "$SCRIPLIR" "$SCRWD" "$SCRMOD" ; do
      if [ -f $RCPATH/init.d/$target ] ; then
        /sbin/insserv -r $RCPATH/init.d/$target
      fi
    done
    for token in "ViPNet" "$SCRIPLIR" "$SCRWD" "$SCRMOD" "$SCRFO" ; do
      cp -f /etc/insserv.conf /tmp/drimomanid;
      sed "/$token/,+0 d" < /tmp/drimomanid > /etc/insserv.conf;
    done
    rm -f /tmp/drimomanid

    echo "# ViPNet services" >> /etc/insserv.conf
    echo "\$vipmod	$SCRMOD" >> /etc/insserv.conf
    echo "\$failover	$SCRFO" >> /etc/insserv.conf
    _log_message "Modified /etc/insserv.conf for ViPNet services support"
    for target in "$SCRMOD" "$SCRFO" ; do
      /sbin/insserv $RCPATH/init.d/$target
    done

  else
    _message "Unable to find /etc/insserv.conf needed by installation system."
    _error_exit "Installation ABORTED." 17
  fi
else
  for lev in 0 1 2 3 4 5 6 ; do
    for target in "$SCRIPLIR" "$SCRWD" "$SCRMOD" "$SCRFO" "$SCRMFTP" "rmiplir" ; do
      rm -f $RCLEVPATH/rc$lev.d/*$target
    done
  done
  for lev in 0 1 6 ; do
    ln -f -s $RCPATH/init.d/$SCRMOD $RCLEVPATH/rc$lev.d/K$MOD_KN$SCRMOD
    log_vipnet_file $RCLEVPATH/rc$lev.d/K$MOD_KN$SCRMOD
    _log_message "Creating symlink $RCLEVPATH/rc$lev.d/K$MOD_KN$SCRMOD -> $RCPATH/init.d/$SCRMOD"
    ln -f -s $RCPATH/init.d/$SCRFO $RCLEVPATH/rc$lev.d/K$FO_KN$SCRFO
    log_vipnet_file $RCLEVPATH/rc$lev.d/K$FO_KN$SCRFO
    _log_message "Creating symlink $RCLEVPATH/rc$lev.d/K$FO_KN$SCRFO -> $RCPATH/init.d/$SCRFO"
  done
  for lev in 2 3 4 5 ; do
    ln -f -s $RCPATH/init.d/$SCRMOD $RCLEVPATH/rc$lev.d/S$MOD_SN$SCRMOD
    log_vipnet_file $RCLEVPATH/rc$lev.d/S$MOD_SN$SCRMOD
    _log_message "Creating symlink $RCLEVPATH/rc$lev.d/S$MOD_SN$SCRMOD -> $RCPATH/init.d/$SCRMOD"
    ln -f -s $RCPATH/init.d/$SCRFO $RCLEVPATH/rc$lev.d/S$FO_SN$SCRFO
    log_vipnet_file $RCLEVPATH/rc$lev.d/S$FO_SN$SCRFO
    _log_message "Creating symlink $RCLEVPATH/rc$lev.d/S$FO_SN$SCRFO -> $RCPATH/init.d/$SCRFO"
  done
fi

#webgui server needs user www-data in group www-data
id -Gn www-data 2>/dev/null | grep www-data > /dev/null
if [ $? -ne 0 ]; then
     _message "Adding user www-data to system"
     groupadd www-data
     useradd -g www-data www-data
fi

mkdir -p $SNMP_DIR
log_vipnet_file $SNMP_DIR
_log_message "Creating directory $SNMP_DIR"

_log_message "Deleting existing man pages"
rm -f $MANPAGE_FILES

_message "Installing man pages"

command -v manpath > /dev/null
if [ $? -ne 0 ]; then
  MANFOLDER=$MANPATH
else
  MANFOLDER=$(manpath)
fi
VIPNET_MAN_PATH=${MANFOLDER%%:*}
if [ ! -d "$VIPNET_MAN_PATH" ]; then
    VIPNET_MAN_PATH="/usr/local/man"
fi
VIPNET_MAN_PATH="$VIPNET_MAN_PATH/ru_RU.UTF-8"
mkdir -p $VIPNET_MAN_PATH
cd $TMP_DISTRIBUTION_DIR
cp -rf manpages/* $VIPNET_MAN_PATH
_log_message "Man pages have been installed at $VIPNET_MAN_PATH"

_log_message "Copy russian locale LC_CTYPE file"
mkdir -p $LIBPATH/locale
cp -r $TMP_DISTRIBUTION_DIR/locale/ru_RU.utf8 $LIBPATH/locale

_log_message "Copy rules.xsd (crg) file "
mkdir -p $LIBPATH/rules_xsd
cp -f $TMP_DISTRIBUTION_DIR/scripts/rules.xsd $LIBPATH/rules_xsd
cp -f $TMP_DISTRIBUTION_DIR/scripts/rules.xsd.crg $LIBPATH/rules_xsd


_message "STEP 8: Saving startup configuration"
# create numbers file
# correct DRVIPLIR_PARAMS remove drviplir_mtudiff and add threads_count

NEW_DRVIPLIR_PARAMS=""

for param in $DRVIPLIR_PARAMS; do
    PAR_NAME=`echo $param |cut -f 1 -d '='`
    if [ $PAR_NAME != drviplir_mtudiff ] ; then
        NEW_DRVIPLIR_PARAMS="$NEW_DRVIPLIR_PARAMS $param"
    fi
done


rm -f $NUMBERS_FILE
echo >$NUMBERS_FILE
echo "IPLIR_VERS_MAJOR="$IPLIR_VERS_MAJOR_NEW >>$NUMBERS_FILE
echo "IPLIR_VERS_MINOR="$IPLIR_VERS_MINOR_NEW >> $NUMBERS_FILE
echo "IPLIR_VERS_SUBMINOR="$IPLIR_VERS_SUBMINOR_NEW >>$NUMBERS_FILE
echo "IPLIR_BUILD_NUM="$IPLIR_BUILD_NUM_NEW >>$NUMBERS_FILE

if [ "${INITD_WAY}" = "sysv" ] ; then
  echo "FO_SN="$FO_SN >>$NUMBERS_FILE
  echo "FO_KN="$FO_KN >>$NUMBERS_FILE
  echo "MOD_SN="$MOD_SN >>$NUMBERS_FILE
  echo "MOD_KN="$MOD_KN >>$NUMBERS_FILE
fi
echo "MOD="$SCRMOD >>$NUMBERS_FILE
echo "IPLIR="$SCRIPLIR >>$NUMBERS_FILE
echo "FO="$SCRFO >>$NUMBERS_FILE
echo "ALG="$SCRALG >>$NUMBERS_FILE
echo "MFTP="$SCRMFTP >>$NUMBERS_FILE
echo "WEBGUI="$SCRWEBGUI >>$NUMBERS_FILE
echo "VIPNET="$SCRVIPNET >>$NUMBERS_FILE
echo "VPNCMD="$SCRVPNCMD >>$NUMBERS_FILE
echo "CMDSHELL="$SCRCMDSHELL >>$NUMBERS_FILE
echo "RCPATH="$RCPATH >>$NUMBERS_FILE
echo "RCLEVPATH="$RCLEVPATH >>$NUMBERS_FILE
if [ "$install_type" = "UPGRADE" ] ; then
  echo "USEWD=YES" >>$NUMBERS_FILE
  if [ "$USEFO" = "YES" ] ;then
      echo "USEFO=YES" >>$NUMBERS_FILE
      echo "FO_PARAMS=\"cluster\"" >>$NUMBERS_FILE
  else
      echo "USEFO=NO" >>$NUMBERS_FILE
      echo "FO_PARAMS=\"single\"" >>$NUMBERS_FILE
  fi
  echo "USEIPLIR="$USEIPLIR >>$NUMBERS_FILE
  if [ -n "$NEW_DRVIPLIR_PARAMS" ] ; then
    echo "DRVIPLIR_PARAMS=\"$NEW_DRVIPLIR_PARAMS\"" >>$NUMBERS_FILE
  else
    echo "DRVIPLIR_PARAMS=\"\"" >>$NUMBERS_FILE
  fi
  if [ -n "$ITCSWD_PARAMS" ] ; then
    echo "ITCSWD_PARAMS=\"$ITCSWD_PARAMS\"" >>$NUMBERS_FILE
  else
    echo "ITCSWD_PARAMS=\"\"" >>$NUMBERS_FILE
  fi
  if [ -n "$ITCSCRPT_PARAMS" ] ; then
    echo "ITCSCRPT_PARAMS=\"$ITCSCRPT_PARAMS\"" >>$NUMBERS_FILE
  else
    echo "ITCSCRPT_PARAMS=\"\"" >>$NUMBERS_FILE
  fi

  fix_old_param

else
  echo "USEWD=YES" >>$NUMBERS_FILE
  echo "USEIPLIR=YES" >>$NUMBERS_FILE
  echo "USEFO=NO" >>$NUMBERS_FILE
  echo "FO_PARAMS=\"single\"" >>$NUMBERS_FILE
  echo "DRVIPLIR_PARAMS=\"$NEW_DRVIPLIR_PARAMS\"" >>$NUMBERS_FILE
  echo "ITCSWD_PARAMS=\"\"" >>$NUMBERS_FILE
  echo "ITCSCRPT_PARAMS=\"\"" >>$NUMBERS_FILE
fi

chmod u=rw,go=r $NUMBERS_FILE
log_vipnet_file $NUMBERS_FILE
_log_message "Create $NUMBERS_FILE"

. $NUMBERS_FILE

#check and set VDSO mode
set_vdso_compat

cd "$INSTALL_DIR"

# finish "UPGRADE" installation
if [ "$install_type" = "UPGRADE" ] ; then
    rm -rf $TMP_DISTRIBUTION_DIR
    cmp_ccc $VIPNET_DIR
    res=$?
    if [ $res -eq "1" ] ; then
        mv -f $VIPNET_DIR/upgrade $VIPNET_DIR/ccc
    fi
#load cryptodriver
    drv="itcscrpt"
    drviface="itcskrniface"
    drv_name=$drv.ko
    drviface_name=$drviface.ko
    /sbin/insmod $MODPATH/$drviface_name >/dev/null
    /sbin/insmod $MODPATH/$drv_name $ITCSCRPT_PARAMS >/dev/null

    test -f $VIPNET_DIR/user/user.xml -a "x$convert_key" != "x1" && HAVE_USER_XML=1
    test -f $VIPNET_DIR/user/objects.xml -a "x$convert_key" != "x1" && HAVE_OBJECTS_XML=1

    # Check iplir configuration
    _message "Checking IPLIR configuration..."
    /sbin/iplircfg --check $PSWFILE >> $LOG_DIR/$INSTALL_LOG 2>&1
    rc=$?
    rm -f /var/run/iplircfg.session
    if [ $rc -ne 0 ]; then
        _message "Failed to check IPLIR configuration, code $rc. See details in \"$LOG_DIR/$INSTALL_LOG\""
        is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
    else
        if [ $convert_key -eq "1"  ]; then
           _log_message "Converted keys from 2.8 to 3.0"
           _log_message "Generated default rules and ip settings xml files"
        fi
    fi

    _check_mftp_conf

    test -f $VIPNET_DIR/user/user.xml -a "x$HAVE_USER_XML" != "x1" && _message "Default firewall rules are created for open and protected network"
    test -f $VIPNET_DIR/user/objects.xml -a "x$HAVE_OBJECTS_XML" != "x1" && _message "Default named objects are created"

    /sbin/rmmod $drv > /dev/null
    /sbin/rmmod $drviface > /dev/null

    #check /etc/iplirnetpsw format
    IPREGEXP="^[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}"
    NETPSWWARN="storing_password_here_is_not_supported_anymore"
    if [ `head -1 $NETPSWFILE` != "$NETPSWWARN" -a `head -1 $NETPSWFILE| grep $IPREGEXP | wc -l` -eq 0 ] ; then
      IP=`cat $NETPSWFILE| grep $IPREGEXP`
      echo $NETPSWWARN > $NETPSWFILE
      echo $IP >> $NETPSWFILE
      _message "Storing password in '$NETPSWFILE' isn't supported anymore! Remove ViPNet user password from '$NETPSWFILE'"
    fi

    if is_continue "Do you want to start ViPNet services automatically now? (y/n)" ; then
        _log_message "Starting ViPNet services"
        vipnet_start
    fi
    _message "Installation has been completed successfully."
    _done_log
    exit 0
fi

_message "STEP 9: Select ViPNet configuration file (*.DST)"
# find and select DST file
DST_DIR=`pwd`
while true
do
  echo "Searching in '$DST_DIR'..."
  get_files_list $DST_MASK
  get_choice $? "CHANGE" "SKIP"
  DST_FILE=`files_list_get_item $?`
  unset_files_list

  if [ "$DST_FILE" = "Change directory" ] ; then
    while true ; do
      echo -n "Please enter full path to ViPNet configuration file directory:"
      read DST_DIR
      if [ ! -d "$DST_DIR" -o -z "$DST_DIR" ] ; then
        echo "Directory '$DST_DIR' doesn't exist!"
        is_continue "Do you want to continue installation? (y/n)" || _exit_install 0
      else
        cd "$DST_DIR" || \
          _error_exit "Unable to change current directory to '$DST_DIR'! Installation ABORTED." 18
        DST_DIR=`pwd`
        break
      fi
    done
  elif [ "$DST_FILE" = "Skip selection" ] ; then
    echo "Please select and unmerge ViPNet configuration file manually! (See documentation)"
    echo "Beware of having 50M of free space for each IP packet log file"
    _log_message "Configuration file not selected"
    _message "Installation completed."
    _done_log
    exit 0
  else
    break
  fi
done

DST_PATH=$DST_DIR/$DST_FILE
_log_message "Selected configuration file: '$DST_PATH'"

_message "STEP 10: Select directory to install ViPNet configuration"
# select ViPNet configuration directory
if [ "$vipnet_installed" = "INSTALLED" ] ; then
  if is_continue "Do you want to overwrite existing ViPNet configuration at '$VIPNET_DIR'? (y/n)" ; then
    rm -rf $VIPNET_DIR/*
    _log_message "Overwriting existing ViPNet configuration at '$VIPNET_DIR'"
  else
    VIPNET_DIR=""
  fi
fi

if [ -z "$VIPNET_DIR" ] ; then
  while true ; do
    echo -n "Please enter directory path to install selected ViPNet configuration (default '$DEFAULT_VIPNET_DIR'):"
    read VIPNET_DIR
    [ -z "$VIPNET_DIR" ] && VIPNET_DIR=$DEFAULT_VIPNET_DIR
    case $VIPNET_DIR in
         /*) ;;
         *) echo "Path should be absolute"
            continue
	    ;;
    esac    
    _log_message "Selected configuration directory: '$VIPNET_DIR'"
    if [ ! -d "$VIPNET_DIR" ] ; then
      mkdir "$VIPNET_DIR" || \
        _error_exit "Unable to create '$VIPNET_DIR'! Installation ABORTED." 19
      log_vipnet_file "$VIPNET_DIR"
      _log_message "Creating configuration directory '$VIPNET_DIR'"
      break
    else
      if ! is_empty_dir "$VIPNET_DIR" ; then
        echo "Directory '$VIPNET_DIR' isn't empty!"
        if is_continue "Do you want to overwrite all data in '$VIPNET_DIR'? (y/n)" ; then
          rm -rf $VIPNET_DIR/*
          _log_message "Overwriting data in '$VIPNET_DIR'"
          break
        fi
      else
        break
      fi
    fi
  done
fi

rm -rf $TMP_DISTRIBUTION_DIR

#load cryptodriver
"$SBINPATH"/make_drivers_prg
drv="itcscrpt"
drviface="itcskrniface"
drv_name=$drv.ko
drviface_name=$drviface.ko
/sbin/insmod $MODPATH/$drviface_name >/dev/null
/sbin/insmod $MODPATH/$drv_name $ITCSCRPT_PARAMS >/dev/null

touch /etc/vipnetsnmpprg
"$SBINPATH"/make_ext_crg /etc/vipnetsnmpprg

_message "STEP 11: Abonent authentication"
# check ViPNet password
rm -f $TMP_PASSWD
while true ; do
  echo -n "Enter your ViPNet password:"
  stty -echo
  read VIPNET_PASSWORD
  stty echo
  echo
  echo $VIPNET_DIR >>$TMP_PASSWD
  echo $VIPNET_PASSWORD >>$TMP_PASSWD

   # unmerge DST file into selected directory
  # this command needs password for reencrypt RNPK set. If Dst does not 
  # include RNPK, any password is accepted
  $SBINPATH/$SCRUNMERGE -p "$VIPNET_PASSWORD"  "$DST_PATH" "$VIPNET_DIR" || \
  _error_exit "Error unmerging '$DST_PATH' to '$VIPNET_DIR'! Installation ABORTED." 20
  _log_message "Install configuration to '$VIPNET_DIR'"

  #if dst forman 2.8 conver keys to format 3.0
  stat_count=`ls "$VIPNET_DIR/d_station"|wc -l`
  if [ $stat_count -le "1" ]; then
      /sbin/iplircfg --check $TMP_PASSWD
      rm -f /var/run/iplircfg.session
  fi

  if /sbin/iplirpasswd -c $TMP_PASSWD ; then
    _log_message "Password is correct"
    rm -f $PSWFILE
    echo $VIPNET_DIR >>$PSWFILE
    echo $VIPNET_PASSWORD >>$PSWFILE
    rm -f $NETPSWFILE
    echo "127.0.0.1" >>$NETPSWFILE
    _log_message "Saving password to '$PSWFILE'"

    /sbin/iplircfg --check $PSWFILE
    rm -f /var/run/iplircfg.session

    TOTAL="0"
    FS_AVAILABLE="0"

    _space_needed TOTAL
    _space_available FS_AVAILABLE

    if [ $FS_AVAILABLE -lt $TOTAL ] ; then
      _message "Not enough free space for additional IP packet log files, you have only $FS_AVAILABLE bytes available"
      /sbin/rmiplir YES
      [ -f $TMP_DIR/$VIPNET_FILES ] && rm -f $TMP_DIR/$VIPNET_FILES
      /sbin/rmmod $drv > /dev/null
      /sbin/rmmod $drviface > /dev/null
      _error_exit "Installation ABORTED." 21
    else
      _log_message "Got enough free space for additional IP packet log files: $TOTAL bytes available"
    fi
    /sbin/rmmod $drv > /dev/null
    /sbin/rmmod $drviface > /dev/null
  else
    if is_continue "Wrong password! Try again? (y/n)" ; then
      rm -f $TMP_PASSWD
      continue
    else
      _log_message "Password incorrect"
      _message "Unable to continue installation"
      if is_continue "Uninstall ViPNet software? (y/n)" ; then
        /sbin/rmiplir YES
        [ -f $TMP_DIR/$VIPNET_FILES ] && rm -f $TMP_DIR/$VIPNET_FILES
        _log_message "Uninstalling ViPNet software..."
      fi
      /sbin/rmmod $drv > /dev/null
	  /sbin/rmmod $drviface > /dev/null
      _error_exit "Installation ABORTED." 21
    fi
  fi
  rm -f $TMP_PASSWD
  break
done

_check_mftp_conf

# finish "INSTALL" installation
if is_continue "Do you want to start ViPNet services automatically now? (y/n)" ; then
  _log_message "Starting ViPNet services"
  vipnet_start
fi
_message "Installation completed."
_done_log
exit 0
