diff -urN linux/include/linux/netdevice.h linux+iplir/include/linux/netdevice.h
--- linux/include/linux/netdevice.h	2004-08-14 09:38:04.000000000 +0400
+++ linux+iplir/include/linux/netdevice.h	2005-03-10 14:53:22.000000000 +0300
@@ -557,6 +557,11 @@
 
 typedef int gifconf_func_t(struct net_device * dev, char __user * bufptr, int len);
 extern int		register_gifconf(unsigned int family, gifconf_func_t * gifconf);
+#ifdef CONFIG_IPLIR
+extern void __attribute__((regparm(0))) set_packet_filter(
+                              struct sk_buff * __attribute__((regparm(0))) (*in_pf)(struct sk_buff *),
+                              struct sk_buff * __attribute__((regparm(0))) (*out_pf)(struct sk_buff *));
+#endif
 static inline int unregister_gifconf(unsigned int family)
 {
 	return register_gifconf(family, NULL);
diff -urN linux/net/core/dev.c linux+iplir/net/core/dev.c
--- linux/net/core/dev.c	2004-08-14 09:36:58.000000000 +0400
+++ linux+iplir/net/core/dev.c	2005-03-10 16:22:58.000000000 +0300
@@ -114,6 +114,11 @@
 #endif	/* CONFIG_NET_RADIO */
 #include <asm/current.h>
 
+#ifdef CONFIG_IPLIR
+struct sk_buff * __attribute__((regparm(0))) (*in_packet_filter)(struct sk_buff *) = NULL;
+struct sk_buff * __attribute__((regparm(0))) (*out_packet_filter)(struct sk_buff *) = NULL;
+EXPORT_SYMBOL(set_packet_filter);
+#endif
 /* This define, if set, will randomly drop a packet when congestion
  * is more than moderate.  It helps fairness in the multi-interface
  * case when one of them is a hog, but it kills performance for the
@@ -1056,6 +1061,16 @@
 	return err;
 }
 
+#ifdef CONFIG_IPLIR
+void __attribute__((regparm(0))) set_packet_filter(
+                       struct sk_buff (* __attribute__((regparm(0))) (*in_pf)(struct sk_buff *skb)),
+                       struct sk_buff (* __attribute__((regparm(0))) (*out_pf)(struct sk_buff *skb)))
+{
+  in_packet_filter = in_pf;
+  out_packet_filter = out_pf;
+}
+#endif
+
 /**
  *	unregister_netdevice_notifier - unregister a network notifier block
  *	@nb: notifier
@@ -1309,6 +1323,14 @@
 	    __skb_linearize(skb, GFP_ATOMIC))
 		goto out_kfree_skb;
 
+#ifdef CONFIG_IPLIR
+    if (out_packet_filter)
+	{
+	  skb = (*out_packet_filter)(skb);
+	  if (!skb)
+	    return NET_XMIT_SUCCESS;
+	}
+#endif
 	/* If packet is not checksummed and device does not support
 	 * checksumming for this protocol, complete checksumming here.
 	 */
@@ -1773,7 +1795,14 @@
 		net_timestamp(&skb->stamp);
 
 	skb_bond(skb);
-
+#ifdef CONFIG_IPLIR
+	if (in_packet_filter)
+	{
+	  skb = (*in_packet_filter)(skb);
+	  if (!skb)
+	    return NET_RX_DROP;
+	}
+#endif
 	__get_cpu_var(netdev_rx_stat).total++;
 
 	skb->h.raw = skb->nh.raw = skb->data;
@@ -1866,7 +1895,8 @@
 
 		dev = skb->dev;
 
-		netif_receive_skb(skb);
+		if (netif_receive_skb(skb) == NET_RX_DROP)
+		  continue;
 
 		dev_put(dev);
 
diff -urN linux/net/Kconfig linux+iplir/net/Kconfig
--- linux/net/Kconfig	2004-08-14 09:36:13.000000000 +0400
+++ linux+iplir/net/Kconfig	2005-03-10 14:53:58.000000000 +0300
@@ -23,6 +23,11 @@
 menu "Networking options"
 	depends on NET
 
+config IPLIR
+	bool "IpLir crypting network driver"
+	---help---
+	Enable this if you want to use ViPNet.
+
 config PACKET
 	tristate "Packet socket"
 	---help---
