#!/bin/sh

LOGGER_FLAGS="-i -p daemon.info -t change_route"

STROUTES_FILE="/etc/failover.routing.ini"

cmd_route ()
{
	local iface="$2"
	# Replace special characters in aliases and vlan-interfaces.
	local escaped_iface="${iface//:/a}"
	escaped_iface="${escaped_iface//./_}"
	if [ $1 = "before"  ] ; then
		ip_env="FAILOVER_${escaped_iface}_NEW_IP"
		eval ipv=\$$ip_env
		cmd="/sbin/route del $ipv"
	fi
	if [ $1 = "after" ] ; then
		ip_env="FAILOVER_${escaped_iface}_OLD_IP"
		eval ipv=\$$ip_env
		cmd="/sbin/route add $ipv dev $iface"
	fi 
	logger $LOGGER_FLAGS "Run command $cmd"
	eval $cmd
}

add_stroutes ()
{
    if [ -s $STROUTES_FILE ] ; then
        logger $LOGGER_FLAGS "Set default and static routes"
        while read line ;  do
            cmd=`echo $line | awk 'NF == 3 {printf "route add -net %s netmask %s gw %s\n", $1, $2, $3}'`
            if [ -n "$cmd" ]; then
                logger $LOGGER_FLAGS "Run command $cmd"
                eval $cmd
            fi
        done < $STROUTES_FILE
    fi
}

make_route ()
{
	FAILOVER_CHANNELS=${FAILOVER_CHANNELS#\'}
	FAILOVER_CHANNELS=${FAILOVER_CHANNELS%\'}
	for dev in $FAILOVER_CHANNELS ; do
		cmd_route $1 $dev
	done
}

if [ ! -z "$1" ] ; then
	if [ "$1" = "before" ] ; then
		CHANGE_MODE="before"
		if [ "$FAILOVER_MODE" = "PASSIVE" ] ; then
			logger $LOGGER_FLAGS "Exit - nothing is done"
			exit 0
		fi 
	fi
	if [ "$1" = "after" ] ; then
		CHANGE_MODE="after"
		if [ "$FAILOVER_MODE" = "ACTIVE" ] ; then
                    add_stroutes
		    exit 0
		fi 
	fi
else
	echo "uncorect usage"
	exit -1;
fi

make_route $CHANGE_MODE

