/*
 *  Template MIB group implementation - vipnet.c
 *
 */

/* include important headers */
#include <config.h>
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

/* needed by util_funcs.h */
#if TIME_WITH_SYS_TIME
# ifdef WIN32
#  include <sys/timeb.h>
# else
#  include <sys/time.h>
# endif
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#if HAVE_WINSOCK_H
#include <winsock.h>
#endif

/* mibincl.h contains all the snmp specific headers to define the
   return types and various defines and structures. */
#include "mibincl.h"

/* header_generic() comes from here */
#include "util_funcs.h"

#include "agent_read_config.h"

/* include our .h file */
#include "vipnet.h"
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#define MAX_STR_LEN 512
#define MAX_IF_N 16
struct vipnet_sys {
    char netid[MAX_STR_LEN];
    char hostid[MAX_STR_LEN];
    char userid[MAX_STR_LEN];
    int uptime;
    int mode;
    int ifn;
};

struct vipnet_if {
    int idx;
    char descr[MAX_STR_LEN];
    int mode;
    int openin;
    int openinbl;
    int openout;
    int openoutbl;
    int cryptin;
    int cryptinbl;
    int cryptout;
    int cryptoutbl;
    struct vipnet_if *next;
};

struct vipnet_mftp {
    unsigned int uptime;
    int mode;
};

struct vipnet_watchdog {
    unsigned int uptime;
};

struct vipnet_failover {
    unsigned int uptime;
    int mode;
};

static struct vipnet_sys sys;
static struct vipnet_if vif[MAX_IF_N];
static struct vipnet_mftp mftp;
static struct vipnet_watchdog watchdog;
static struct vipnet_failover failover;

int i;
FILE *fp;
char line [MAX_STR_LEN];


   /*
    *  Certain objects can be set via configuration file directives.
    *  These variables hold the values for such objects, as they need to
    *   be accessible to both the config handlers, and the callback routine.
    */
#define VIPNET_STR_LEN	300
#define VIPNET_STR_DEFAULT	"ViPNet"
int	vipnet_int = 42;
char	vipnet_str[ VIPNET_STR_LEN ];

	/* Forward declarations for the config handlers */
void vipnet_parse_config_vipnetint( const char *token, char *cptr );
void vipnet_parse_config_vipnetstr( const char *token, char *cptr );
void vipnet_free_config_vipnetint( void );
void vipnet_free_config_vipnetstr( void );


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/

    /*
     * This array structure defines a representation of the
     *  MIB being implemented.
     *
     * The type of the array is 'struct variableN', where N is
     *  large enough to contain the longest OID sub-component
     *  being loaded.  This will normally be the maximum value
     *  of the fifth field in each line.  In this case, the second
     *  and third entries are both of size 2, so we're using
     *  'struct variable2'
     *
     * The supported values for N are listed in <agent/var_struct.h>
     *  If the value you need is not listed there, simply use the
     *  next largest that is.
     *
     * The format of each line is as follows
     *  (using the first entry as an example):
     *	    1: VIPNETSTRING:
     *		The magic number defined in the vipnet header file.
     *		This is passed to the callback routine and is used
     *		  to determine which object is being queried.
     *	    2: ASN_OCTET_STR:
     *		The type of the object.
     *		Valid types are listed in <snmp_impl.h>
     *	    3: RONLY (or RWRITE):
     *		Whether this object can be SET or not.
     *	    4: var_vipnet:
     *		The callback routine, used when the object is queried.
     *		This will usually be the same for all objects in a module
     *		  and is typically defined later in this file.
     *	    5: 1:
     *		The length of the OID sub-component (the next field)
     *	    6: {1}:
     *		The OID sub-components of this entry.
     *		In other words, the bits of the full OID that differ
     *		  between the various entries of this array.
     *		This value is appended to the common prefix (defined later)
     *		  to obtain the full OID of each entry.
     */


struct variable4 vipnet_variables[] = {
#define VIPNETNETID            	1
    { VIPNETNETID,    		ASN_OCTET_STR, 	RONLY, var_vipnet, 2, {1,1}},
#define VIPNETHOSTID           	2
    { VIPNETHOSTID,   		ASN_OCTET_STR, 	RONLY, var_vipnet, 2, {1,2}},
#define VIPNETUSERID           	3
    { VIPNETUSERID,  		ASN_OCTET_STR, 	RONLY, var_vipnet, 2, {1,3}},
#define VIPNETUPTIME           	4
    { VIPNETUPTIME, 		ASN_TIMETICKS, 	RONLY, var_vipnet, 2, {1,4}},

#define VIPNETIFNUMBER         	5
    { VIPNETIFNUMBER, 		ASN_INTEGER, 	RONLY, var_vipnet, 2, {2,1}},
#define VIPNETIFINDEX          	6
    { VIPNETIFINDEX,  		ASN_INTEGER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,1}},
#define VIPNETIFDESCR          	7
    { VIPNETIFDESCR,  		ASN_OCTET_STR, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,2}},
#define VIPNETIFMODE           	8
    { VIPNETIFMODE,  		ASN_INTEGER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,3}},
#define VIPNETIFOPENIN         	9
    { VIPNETIFOPENIN,  		ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,4}},
#define VIPNETIFOPENINBLOCK     10
    { VIPNETIFOPENINBLOCK,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,5}},
#define VIPNETIFOPENOUT         11
    { VIPNETIFOPENOUT,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,6}},
#define VIPNETIFOPENOUTBLOCK    12
    { VIPNETIFOPENOUTBLOCK,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,7}},
#define VIPNETIFCRYPTIN         13
    { VIPNETIFCRYPTIN,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,8}},
#define VIPNETIFCRYPTINBLOCK    14
    { VIPNETIFCRYPTINBLOCK,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,9}},
#define VIPNETIFCRYPTOUT        15
    { VIPNETIFCRYPTOUT,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,10}},
#define VIPNETIFCRYPTOUTBLOCK   16
    { VIPNETIFCRYPTOUTBLOCK,  	ASN_COUNTER, 	RONLY, var_vipnet_ifentry, 4, {2,2,1,11}},

};


    /*
     * This array defines the OID of the top of the mib tree that we're
     *  registering underneath.
     * Note that this needs to be the correct size for the OID being 
     *  registered, so that the length of the OID can be calculated.
     *  The format given here is the simplest way to achieve this.
     */
oid vipnet_variables_oid[] = { 1, 3, 6, 1, 4, 1, 10812, 1 };

int send_vipnet_trap( int status)
{
	oid vipnet_trap_oid[]={ 1, 3, 6, 1, 4, 1, 10812, 1, 0, 1};
	int vipnet_trap_oid_len;
	static struct variable_list var_trap;
	oid objid_snmptrap[] = { 1,3,6,1,6,3,1,1,4,1,0}; /* snmpTrapIOD.0 */

	vipnet_trap_oid[9] = status;
	vipnet_trap_oid_len = sizeof( vipnet_trap_oid)/sizeof(oid);
	var_trap.next_variable = NULL; /* No more variables after */
	var_trap.name = objid_snmptrap; /* snmpTrapIOD.0 */
	var_trap.name_length = sizeof(objid_snmptrap)/sizeof(oid); /* number of sub-id s */
	var_trap.type = ASN_OBJECT_ID;
	var_trap.val.objid = vipnet_trap_oid;
	var_trap.val_len = sizeof(vipnet_trap_oid);

        DEBUGMSGTL(("vipnet","send_vipnet_trap: %d\n",status));
        snmp_log(LOG_ERR,"send_vipnet_trap: status %d\n",status);
	send_v2trap(&var_trap);
	return 0;
};

static time_t last_vipnet = 0, last_mftp = 0, last_watchdog = 0,
              last_failover = 0;
static int isup_vipnet = 1, isup_mftp = 1, isup_watchdog = 1,
           isup_failover = 1;
static int last_mode = -1;
static unsigned int failover_flags = 0, iplir_flags = 0, mftp_flags = 0;

static void vipnetupdownstatus(unsigned int registrationNumber, void *clientarg)
{
  static unsigned char first_call = 1;
	snmp_log(LOG_ERR, "==========================================================================\n");
	snmp_log(LOG_ERR, "(vipnetupdown): time now = %d.\n", time(NULL));

  // first check failover status
	if ( get_failover_stat () == 1 ) 
    snmp_log(LOG_ERR,"call get_failover_stat from vipnetupdownstatus return error.\n");

  if (last_mode == -1)
    last_mode = failover.mode;

	snmp_log(LOG_ERR, "(vipnetupdown): failover uptime= %d lasttime = %d "
                    "isup = %d mode = %d last_mode = %d.\n", 
                    failover.uptime, last_failover, isup_failover,
                    failover.mode, last_mode);

	if (failover.uptime > last_failover)
	{
		last_failover = failover.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: GOOD failover "
                     "time: %d\n", last_failover);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                            "failover time: %d\n", last_failover));
		if (isup_failover == FALSE)
		{
		    isup_failover = TRUE;		
		    if (!first_call)
		    {
          snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                           "failover is up\n");
          DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                      "failover is up\n"));
          if (failover.mode == 1)
            send_vipnet_trap(TRAP_FAILOVER_UP);
		    }
		}
    else if (failover.mode == 1 && last_mode != failover.mode)
    {
      if ( !(iplir_flags & IPLIR_SWITCHED) && isup_vipnet )
       failover_flags |= IPLIR_SWITCHED;

      if ( !(mftp_flags & MFTP_SWITCHED) && isup_mftp )
       failover_flags |= MFTP_SWITCHED;

      send_vipnet_trap(TRAP_FAILOVER_SWITCH);
    }

    last_mode = failover.mode;
	}
  else
	{
		last_failover = failover.uptime;
    last_mode = failover.mode;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: BAD failover "
                     "time: %d\n", last_failover);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                "failover time: %d\n", last_failover));
		if (isup_failover == TRUE)
		{
      snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                       "failover is down\n");
      DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                  "failover is down\n"));
      isup_failover = FALSE;
      if (failover.mode == 1 || failover.mode == 2)
        send_vipnet_trap(TRAP_FAILOVER_DOWN);
		}
	} 	

	if ( get_vipnet_stat () == 1 ) 
	{
	    snmp_log(LOG_ERR,"call get_vipnet_stat from vipnetupdownstatus return error.\n");
	}
	if ( get_mftp_stat () == 1 ) 
	{
	    snmp_log(LOG_ERR,"call get_mftp_stat from vipnetupdownstatus return error.\n");
	}
	if ( get_watchdog_stat () == 1 ) 
	{
	    snmp_log(LOG_ERR,"call get_watchdog_stat from vipnetupdownstatus return error.\n");
	}
	
	snmp_log(LOG_ERR, "(vipnetupdown): iplir uptime= %d lasttime = %d "
                    "isup = %d mode = %d.\n", sys.uptime, last_vipnet, 
                    isup_vipnet, sys.mode);
	if (sys.uptime > last_vipnet)
	{
		last_vipnet = sys.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: GOOD iplir "
                     "time: %d\n", last_vipnet);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                "iplir time: %d\n", last_vipnet));
		if (isup_vipnet == FALSE)
		{
      isup_vipnet = TRUE;		
      if (!first_call)
      {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "iplir is up\n");
        DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                    "iplir is up\n"));
        if (sys.mode == 1)
          send_vipnet_trap(TRAP_VIPNET_UP);
      }
		}		    
	}
  else
	{
		last_vipnet = sys.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: BAD iplir "
                     "time: %d\n", last_vipnet);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                "iplir time: %d\n", last_vipnet));
		if (isup_vipnet == TRUE)
		{
      if ( (failover_flags & IPLIR_SWITCHED) ||
           (isup_failover && failover.mode == 2) )
      {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "iplir is switched\n");
        failover_flags &= ~IPLIR_SWITCHED;
        iplir_flags |= IPLIR_SWITCHED;
      }
      else
      {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "iplir is down\n");
        DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                    "iplir is down\n"));
        isup_vipnet = FALSE;
  
        if (sys.mode == 1)
          send_vipnet_trap(TRAP_VIPNET_DOWN);
      }
		}
	} 	

	snmp_log(LOG_ERR, "(vipnetupdown): mftp uptime= %d lasttime = %d "
                    "isup = %d mode = %d.\n", mftp.uptime, 
                    last_mftp, isup_mftp, mftp.mode);
	if (mftp.uptime > last_mftp)
	{
		last_mftp = mftp.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: GOOD mftp "
                     "time: %d\n", last_mftp);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                "mftp time: %d\n", last_mftp));
		if (isup_mftp == FALSE)
		{
		    isup_mftp = TRUE;
		    if (!first_call)			
		    {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "mftp is up (mode=%d)\n", mftp.mode);
        DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                    "mftp is up (mode=%d)\n", mftp.mode));
        if (mftp.mode == 1)
	        send_vipnet_trap(TRAP_MFTP_UP);
	    }			  
		}
	}
  else
	{
		last_mftp = mftp.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: BAD mftp "
                     "time: %d\n", last_mftp);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                "mftp time: %d\n", last_mftp));
		if (isup_mftp == TRUE)
		{
      if ( (failover_flags & MFTP_SWITCHED) ||
           (isup_failover && failover.mode == 2)
         )
      {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "mftp is switched\n");
        failover_flags &= ~MFTP_SWITCHED;
        mftp_flags |= MFTP_SWITCHED;
      }
      else
      {
        snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: "
                         "mftp is down (mode=%d)\n", mftp.mode);
        DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                    "mftp is down (mode=%d)\n", mftp.mode));
  			isup_mftp = FALSE;

        if (mftp.mode == 1)
          send_vipnet_trap(TRAP_MFTP_DOWN);
      }
		}
	} 	

	snmp_log(LOG_ERR, "(vipnetupdown): watchdog uptime= %d lasttime = %d "
                          "isup = %d.\n", watchdog.uptime, last_watchdog,
                           isup_watchdog);
	if (watchdog.uptime > last_watchdog)
	{
		last_watchdog = watchdog.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: GOOD watchdog "
                                  "time: %d\n", last_watchdog);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                            "watchdog time: %d\n", last_watchdog));
		if (isup_watchdog == FALSE)
		{
		    isup_watchdog = TRUE;		
		    if (!first_call)
		    {		    		
			snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus",
                                         "watchdog is up\n");
			DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                                    "watchdog is up\n"));
	    		send_vipnet_trap(TRAP_WATCHDOG_UP);
		    }			
		}
	}
  else
	{
		last_watchdog = watchdog.uptime;
		snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus: BAD watchdog "
                                 "time: %d\n", last_watchdog);
		DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                            "watchdog time: %d\n", last_watchdog));
		if (isup_watchdog == TRUE)
		{
      snmp_log(LOG_ERR,"vipnet: vipnetupdownstatus",
                       "watchdog is down\n");
      DEBUGMSGTL(("vipnet: vipnetupdownstatus",
                                    "watchdog is down\n"));
			isup_watchdog = FALSE;
	    		send_vipnet_trap(TRAP_WATCHDOG_DOWN);
		}
	} 	

	first_call = 0;
}

    /*
     * This function is called at the time the agent starts up
     *  to do any initializations that might be required.
     *
     * In theory it is optional and can be omitted if no
     *  initialization is needed.  In practise, every module
     *  will need to register itself (or the objects being
     *  implemented will not appear in the MIB tree), and this
     *  registration is typically done here.
     *
     * If this function is added or removed, you must re-run
     *  the configure script, to detect this change.
     */
void init_vipnet(void)
{
    /*
     * Register ourselves with the agent to handle our mib tree.
     * The arguments are:
     *    descr:   A short description of the mib group being loaded.
     *    var:     The variable structure to load.
     *			(the name of the variable structure defined above)
     *    vartype: The type of this variable structure
     *    theoid:  The OID pointer this MIB is being registered underneath.
     */
  REGISTER_MIB("vipnet", vipnet_variables, variable4, vipnet_variables_oid);


    /*
     *  Register config handlers for the two objects that can be set
     *   via configuration file directive.
     *  Also set a default value for the string object.  Note that the
     *   vipnet integer variable was initialised above.
     */
  strcpy( vipnet_str, VIPNET_STR_DEFAULT );

  snmpd_register_config_handler( "vipnetint",
			vipnet_parse_config_vipnetint,
			vipnet_free_config_vipnetint,
			"vipnetint value" );
  snmpd_register_config_handler( "vipnetstr",
			vipnet_parse_config_vipnetstr,
			vipnet_free_config_vipnetstr,
			"vipnetstr value" );
  snmpd_register_config_handler( "vipnetstring",
			vipnet_parse_config_vipnetstr,
			vipnet_free_config_vipnetstr,
			"vipnetstring value" );

    /*
     * One common requirement is to read values from the kernel.
     * This is usually initialised here, to speed up access when the
     *  information is read in, as a response to an incoming request.
     *
     * This module doesn't actually use this mechanism,
     * so this call is commented out here.
     */
  sys.netid[0]=0;
  sys.hostid[0]=0;
  sys.userid[0]=0;
  sys.uptime=0;
  sys.ifn=0;
  for (i=0;i<MAX_IF_N;i++){
     vif[i].idx=i;
     vif[i].descr[0]=0;
     vif[i].mode=0;
     vif[i].openin=0;
     vif[i].openinbl=0;
     vif[i].openout=0;
     vif[i].openoutbl=0;
     vif[i].cryptin=0;
     vif[i].cryptinbl=0;
     vif[i].cryptout=0;
     vif[i].cryptoutbl=0;

  };
  mftp.uptime = 0;
  watchdog.uptime = 0;
  failover.uptime = 0;

  /* We must read UPTIME values here
  ** in order to avoid wrong trap's to
  ** SNMP-managment system. -santa
  */
  if ( get_vipnet_stat () == 1 ) 
  {
    snmp_log(LOG_ERR,"call get_vipnet_stat from init_vipnet return error.\n");
    DEBUGMSGTL(("vipnet","call get_vipnet_stat from init_vipnet return error.\n"));
  }
  if ( get_mftp_stat () == 1 ) 
  {
    snmp_log(LOG_ERR,"call get_mftp_stat from init_vipnet return error.\n");
    DEBUGMSGTL(("vipnet","call get_mftp_stat from init_vipnet return error.\n"));
  }
  if ( get_watchdog_stat () == 1 ) 
  {
    snmp_log(LOG_ERR,"call get_watchdog_stat from init_vipnet return error.\n");
    DEBUGMSGTL(("vipnet","call get_watchdog_stat from init_vipnet return error.\n"));
  }
  if ( get_failover_stat () == 1 ) 
  {
    snmp_log(LOG_ERR,"call get_failover_stat from init_vipnet return error.\n");
    DEBUGMSGTL(("vipnet","call get_failover_stat from init_vipnet return error.\n"));
  }
  
  last_vipnet = sys.uptime;
  isup_vipnet = FALSE;
  last_mftp = mftp.uptime;
  isup_mftp = FALSE;
  last_watchdog = watchdog.uptime;
  isup_watchdog = FALSE;
  last_failover = failover.uptime;
  isup_failover = FALSE;

  /* This is the one that runs update_stats every POLL_INTERVAL seconds */
  if (snmp_alarm_register(POLL_INTERVAL, SA_REPEAT, vipnetupdownstatus, NULL) == 0)
    {
      snmp_log(LOG_ERR, "(init_vipnet): snmp_alarm_register failed, cannot service requests.\n");
    }

}

	/*********************
	 *
	 *  Configuration file handling functions
	 *
	 *********************/

void
vipnet_parse_config_vipnetint( const char *token, char *cptr )
{
    vipnet_int = atoi( cptr );
}

void
vipnet_parse_config_vipnetstr( const char *token, char *cptr )
{
		/*
		 * Make sure the string fits in the space allocated for it.
		 */
    if ( strlen( cptr ) < VIPNET_STR_LEN )
	strcpy( vipnet_str, cptr );
    else {
		/*
		 * Truncate the string if necessary.
		 * An alternative approach would be to log an error,
		 *  and discard this value altogether.
		 */
	strncpy( vipnet_str, cptr, VIPNET_STR_LEN-3 );
	vipnet_str[ VIPNET_STR_LEN-3 ] = 0;
	strcat(  vipnet_str, "..." );
    }
}

	/*  We don't need to do anything special when closing down */
void
vipnet_free_config_vipnetint( void ) {}

void
vipnet_free_config_vipnetstr( void ) {}


int get_vipnet_stat (void) 
{
  int j;
  DEBUGMSGTL(("vipnet","get_vipnet_stat entered\n"));

  fp=fopen(VIPNET_STAT_FILE, "rt");
  if (! (fp) )
  {
    DEBUGMSGTL(("vipnet","get_vipnet_stat can't open file\n"));
    return 1;
  }

  if (read_lock(fileno(fp)) != 0)
  {
    DEBUGMSGTL(("vipnet","get_vipnet_stat: read_lock failed\n"));
    fclose(fp);
    return 1;
  }

  line == fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf (line,"%s %s %s %d %d %d", sys.netid, sys.hostid, sys.userid, 
              &sys.uptime, &sys.mode, &sys.ifn);
  if (j<6){

    DEBUGMSGTL(("vipnet","get_vipnet_stat WRONG STAT DATA: %s\n",line));

    read_unlock(fileno(fp));
    fclose (fp);
    return 1;
  }

  for (i=1;i<=sys.ifn;i++)
  {
    line == fgets (line, sizeof(line), fp);
    if ( line[strlen(line)-1] == '\n' ) line[strlen(line)-1]='\0';
    j=sscanf (line, "%s %d %d %d %d %d %d %d %d",
                   vif[i].descr,
                  &vif[i].openin,   &vif[i].openinbl,
                  &vif[i].openout,  &vif[i].openoutbl,
                  &vif[i].cryptin,  &vif[i].cryptinbl,
                  &vif[i].cryptout, &vif[i].cryptoutbl);
    vif[i].mode = 0;
    if (j<9)
    {
        DEBUGMSGTL(("vipnet","get_vipnet_stat WRONG IF DATA: %s\n",line));

        read_unlock(fileno(fp));
        fclose (fp);
        return 1;
    }

  }

  read_unlock(fileno(fp));
  fclose (fp);
  return 0;
}

int get_mftp_stat(void) 
{
  int j;
  int mode;
  DEBUGMSGTL(("vipnet","get_mftp_stat entered\n"));

  fp = fopen(MFTP_STAT_FILE, "rt");
  if (!fp)
  {
    DEBUGMSGTL(("vipnet","get_mftp_stat cant open file\n"));
    return 1;
  }

  if (read_lock(fileno(fp)) != 0)
  {
    DEBUGMSGTL(("vipnet","get_mftp_stat: read_lock failed\n"));
    fclose(fp);
    return 1;
  }

  fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf(line,"%u", &mftp.uptime);
  if (j < 1)
  {
    DEBUGMSGTL(("vipnet","get_mftp_stat WRONG STAT UPTIME DATA: %s\n",line));

    read_unlock(fileno(fp));
    fclose (fp);
    return 1;
  }

  fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf(line,"%d", &mode);
  if (j < 1)
  {
    DEBUGMSGTL(("vipnet","get_mftp_stat WRONG STAT DATA: %s\n",line));
  }
  else
    mftp.mode = mode;

  read_unlock(fileno(fp));
  fclose (fp);
  return 0;
}

int get_watchdog_stat(void) 
{
  int j;
  DEBUGMSGTL(("vipnet","get_watchdog_stat entered\n"));

  fp = fopen(WATCHDOG_STAT_FILE, "rt");
  if (!fp)
  {
    DEBUGMSGTL(("vipnet","get_watchdog_stat cant open file\n"));
    return 1;
  }

  if (read_lock(fileno(fp)) != 0)
  {
    DEBUGMSGTL(("vipnet","get_watchdog_stat: read_lock failed\n"));
    fclose(fp);
    return 1;
  }

  line == fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf(line,"%u", &watchdog.uptime);
  if (j < 1)
  {
    DEBUGMSGTL(("vipnet","get_watchdog_stat WRONG STAT DATA: %s\n",line));

    read_unlock(fileno(fp));
    fclose (fp);
    return 1;
  }

  read_unlock(fileno(fp));
  fclose (fp);
  return 0;
}

int get_failover_stat(void) 
{
  int j;
  int mode;
  DEBUGMSGTL(("vipnet","get_failover_stat entered\n"));

  fp = fopen(FAILOVER_STAT_FILE, "rt");
  if (!fp)
  {
    DEBUGMSGTL(("vipnet","get_failover_stat cant open file\n"));
    return 1;
  }

  if (read_lock(fileno(fp)) != 0)
  {
    DEBUGMSGTL(("vipnet","get_failover_stat: read_lock failed\n"));
    fclose(fp);
    return 1;
  }

  fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf(line,"%u", &failover.uptime);
  if (j < 1)
  {
    DEBUGMSGTL(("vipnet","get_failover_stat WRONG STAT UPTIME DATA: %s\n",line));

    read_unlock(fileno(fp));
    fclose (fp);
    return 1;
  }

  fgets (line, sizeof(line), fp);
  if ( line[strlen(line)-1] == '\n' ) 
      line[strlen(line)-1]='\0';
  j = sscanf(line,"%d", &mode);
  if (j < 1)
  {
    DEBUGMSGTL(("vipnet","get_failover_stat WRONG STAT DATA: %s\n",line));
  }
  else
    failover.mode = mode;

  read_unlock(fileno(fp));
  fclose (fp);
  return 0;
}


	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

    /*
     * Define the callback function used in the vipnet_variables structure.
     * This is called whenever an incoming request refers to an object
     *  within this sub-tree.
     *
     * Four of the parameters are used to pass information in.
     * These are:
     *	  vp	  The entry from the 'vipnet_variables' array for the
     *		   object being queried.
     *	  name	  The OID from the request.
     *	  length  The length of this OID.
     *	  exact	  A flag to indicate whether this is an 'exact' request
     *		   (GET/SET) or an 'inexact' one (GETNEXT/GETBULK).
     *
     * Four of the parameters are used to pass information back out.
     * These are:
     *	  name	   The OID being returned.
     *	  length   The length of this OID.
     *	  var_len  The length of the answer being returned.
     *    write_method   A pointer to the SET function for this object.
     *
     * Note that name & length serve a dual purpose in both roles.
     */

u_char	*
var_vipnet(struct variable *vp,
	    oid *name,
	    size_t *length,
	    int exact,
	    size_t *var_len,
	    WriteMethod **write_method)
{
    /*
     *  The result returned from this function needs to be a pointer to
     *    static data (so that it can be accessed from outside).
     *  Define suitable variables for any type of data we may return.
     */
  static char string[MAX_STR_LEN];	/* for VIPNETSTRING   */
  static oid  oid_ret[16];		/* for VIPNETOBJECTID */
  static long long_ret;			/* for everything else */
  static oid objid[MAX_OID_LEN];
  static struct counter64 c64;


  
    /*
     * Before returning an answer, we need to check that the request
     *  refers to a valid instance of this object.  The utility routine
     *  'header_generic' can be used to do this for scalar objects.
     *
     * This routine 'header_simple_table' does the same thing for "simple"
     *  tables. (See the AGENT.txt file for the definition of a simple table).
     *
     * Both these utility routines also set up default values for the
     *  return arguments (assuming the check succeeded).
     * The name and length are set suitably for the current object,
     *  var_len assumes that the result is an integer of some form,
     *  and write_method assumes that the object cannot be set.
     *
     * If these assumptions are correct, this callback routine simply
     * needs to return a pointer to the appropriate value (using 'long_ret').
     * Otherwise, 'var_len' and/or 'write_method' should be set suitably.
     */
  DEBUGMSGTL(("vipnet","var_vipnet entered\n"));
  if (header_generic(vp, name, length, exact, var_len, write_method) == MATCH_FAILED)
    return NULL;


    /*
     * Many object will need to obtain data from the operating system in
     *  order to return the appropriate value.  Typically, this is done
     *  here - immediately following the 'header' call, and before the
     *  switch statement. This is particularly appropriate if a single 
     *  interface call can return data for all the objects supported.
     *
     * This example module does not rely on external data, so no such
     *  calls are needed in this case.  
     */

  DEBUGMSGTL(("vipnet","var_vipnet header_generic end\n"));

  if ( get_vipnet_stat () == 1 ) 
      return NULL;

  DEBUGMSGTL(("vipnet","get_vipnet_stat end\n"));

    /*
     * Now use the magic number from the variable pointer 'vp' to
     *  select the particular object being queried.
     * In each case, one of the static objects is set up with the
     *  appropriate information, and returned mapped to a 'u_char *'
     */
  switch (vp->magic){
    case VIPNETNETID:
      sprintf( string,"%s", sys.netid);
      *var_len = strlen(string);
      return (u_char *) string;

    case VIPNETHOSTID:
      sprintf(string, "%s", sys.hostid);
      *var_len = strlen(string);
      return (u_char *) string;

    case VIPNETUSERID:
      sprintf(string, "%s", sys.userid);
      *var_len = strlen(string);
      return (u_char *) string;

    case VIPNETUPTIME:
      long_ret = sys.uptime;
      return (u_char *) &long_ret;
		  
    case VIPNETIFNUMBER:
      long_ret = sys.ifn;
      return (u_char *) &long_ret;

    default:
		/*
		 *  This will only be triggered if there's a problem with
		 *   the coding of the module.  SNMP requests that reference
		 *   a non-existant OID will be directed elsewhere.
		 *  If this branch is reached, log an error, so that
		 *   the problem can be investigated.
		 */
      DEBUGMSGTL(("snmpd", "unknown sub-id %d in vipnets/var_vipnet\n", vp->magic));
  }
  /*
   * If we fall through to here, fail by returning NULL.
   * This is essentially a continuation of the 'default' case above.
   */
  return NULL;
}


static int
header_vipnet_ifentry(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method,
	    int     ifcount)
{
#define IFENTRY_NAME_LENGTH     12
    oid newname[MAX_OID_LEN];
    int result;
    DEBUGMSGTL(("vipnet/interfaces", "var_vipnet_ifentry: \n"));
    DEBUGMSGOID(("vipnet/interfaces", name, *length));
    DEBUGMSGTL(("vipnet/interfaces", " EXACT  %d\n", exact));
	    
    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
    for(i = 1; i <= ifcount ; i++)
    {

DEBUGMSGTL(("vipnet/interfaces", "CHECK i=%d ifcount=%d \n",i,ifcount ));

        newname[IFENTRY_NAME_LENGTH] = (oid)i;
        result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
        if ((exact && (result == 0)) || (!exact && (result < 0)))
	    break;
    }
    if (i > ifcount) {

DEBUGMSGTL(("vipnet/interfaces", "... index out of range  i=%d\n",i));

        return MATCH_FAILED;
    }
			

    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    *length = vp->namelen + 1;
    *write_method = 0;
    *var_len = sizeof(long);    /* default to 'long' results */
	    
DEBUGMSGTL(("vipnet/interfaces", "... get I/F stats \n"));
DEBUGMSGOID(("vipnet/interfaces OID", name, *length));
DEBUGMSGTL(("vipnet/interfaces", "header_vipnet_ifentry end","\n"));

    return i;
}

/*
 * var_ifentry():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_vipnet above.
 */
unsigned char *
var_vipnet_ifentry(struct variable *vp,
            oid     *name,
            size_t  *length,
            int     exact,
            size_t  *var_len,
            WriteMethod **write_method)
{


  /* variables we may use later */
  static long long_ret;
  static unsigned char string[SPRINT_MAX_LEN];
  static oid objid[MAX_OID_LEN];
  static struct counter64 c64;
  int interface;
  if ( get_vipnet_stat() == 1) return NULL;

  /*
   * This assumes that the table is a 'simple' table.
   *    See the implementation documentation for the meaning of this.
   *    You will need to provide the correct value for the TABLE_SIZE parameter
   *
   * If this table does not meet the requirements for a simple table,
   *    you will need to provide the replacement code yourself.
   *    Mib2c is not smart enough to write this for you.
   *    Again, see the implementation documentation for what is required.
   */
  interface = header_vipnet_ifentry(vp,name,length,exact,var_len,write_method, sys.ifn);
  if (interface <= 0 )
     return NULL;
  
  /*
   * this is where we do the value assignments for the mib results.
   */

  switch(vp->magic) {


    case VIPNETIFINDEX:
DEBUGMSGTL(("vipnet", "return VIPNETIFINDEX from var_vipnet_ifentry %d count=%d\n", interface, sys.ifn));
        long_ret = interface;
        return (unsigned char *) &long_ret;

    case VIPNETIFDESCR:
        sprintf(string,"%s", vif[interface].descr);
DEBUGMSGTL(("vipnet", "return VIPNETIFDESCR from var_vipnet_ifentry %s \n", string));
        *var_len = strlen(string);
        return (unsigned char *) string;

    case VIPNETIFMODE:
        long_ret = vif[interface].mode;
DEBUGMSGTL(("vipnet", "return VIPNETIFMODE from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFOPENIN:
        long_ret = vif[interface].openin;
DEBUGMSGTL(("vipnet", "return VIPNETIFOPENIN from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFOPENINBLOCK:
        long_ret = vif[interface].openinbl;
DEBUGMSGTL(("vipnet", "return VIPNETIFOPENINBLOCK from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFOPENOUT:
        long_ret = vif[interface].openout;
DEBUGMSGTL(("vipnet", "return VIPNETIFOPENOUT from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFOPENOUTBLOCK:
        long_ret = vif[interface].openoutbl;
DEBUGMSGTL(("vipnet", "return VIPNETIFOPENOUTBLOCK from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFCRYPTIN:
        long_ret = vif[i].cryptin;
DEBUGMSGTL(("vipnet", "return VIPNETIFCRYPTIN from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFCRYPTINBLOCK:
        long_ret = vif[interface].cryptinbl;
DEBUGMSGTL(("vipnet", "return VIPNETIFCRYPTINBLOCK from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFCRYPTOUT:
        long_ret = vif[interface].cryptout;
DEBUGMSGTL(("vipnet", "return VIPNETIFCRYPTOUT from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;

    case VIPNETIFCRYPTOUTBLOCK:
        long_ret = vif[interface].cryptoutbl;
DEBUGMSGTL(("vipnet", "return VIPNETIFCRYPTOUTBLOCK from var_vipnet_ifentry %d \n", long_ret));
        return (unsigned char *) &long_ret;


    default:
        DEBUGMSGTL(("snmpd", "unknown sub-id %d in vipnets/var_vipnet\n", vp->magic));
  }
  return NULL;
}





	/*********************
	 *
	 *  Writeable object SET handling routines
	 *
	 *********************/

int
write_vipnettrap(int	action,
		 u_char	*var_val,
		 u_char	var_val_type,
		 size_t	var_val_len,
		 u_char	*statP,
		 oid	*name,
		 size_t	name_len)
{
    long intval;

    DEBUGMSGTL(("vipnet","write_vipnettrap entered: action=%d\n",action));
    switch ( action ) {
	case RESERVE1:
			/*
			 *  The only acceptable value is the integer 1
			 */
	    if (var_val_type != ASN_INTEGER ) {
		DEBUGMSGTL(("vipnet", "%x not integer type\n", var_val_type));
		return SNMP_ERR_WRONGTYPE;
	    }
	    if (var_val_len > sizeof(long)) {
		DEBUGMSGTL(("vipnet", "wrong length %x\n", var_val_len));
		return SNMP_ERR_WRONGLENGTH;
	    }

	    intval = *((long *) var_val);
	    if  ( intval != 1 ) {
		DEBUGMSGTL(("vipnet", "wrong value %x\n", intval));
		return SNMP_ERR_WRONGVALUE;
	    }
	    break;

	case RESERVE2:
			/* No resources are required.... */
	    break;

	case FREE:
			/* ... so no resources need be freed */
	    break;

	case ACTION:
			/*
			 *  Having triggered the sending of a trap,
			 *   it would be impossible to revoke this,
			 *   so we can't actually invoke the action here.
			 */
	    break;

	case UNDO:
			/*  We haven't done anything yet,
				so there's nothing to undo */
	    break;

	case COMMIT:
			/*
			 *  Everything else worked, so it's now safe
			 *   to trigger the trap.
			 *  Note that this is *only* acceptable since
			 *   the trap sending routines are "failsafe".
			 *  (In fact, they can fail, but they return no
			 *   indication of this, which is the next best thing!)
			 */
            DEBUGMSGTL(("vipnet","write_vipnettrap sending the trap\n"));
	          send_easy_trap( SNMP_TRAP_ENTERPRISESPECIFIC, 99 );
            DEBUGMSGTL(("vipnet","write_vipnettrap trap sent\n"));
	    break;

    }
    return SNMP_ERR_NOERROR;
}

int read_lock(int fd)
{
	struct flock lock;
  lock.l_type = F_RDLCK;      /* Read locking */
  lock.l_start = 0;		        /* Offset where lock begins */
  lock.l_whence = SEEK_SET;   /* from current file-seek */
  lock.l_len = 0;	            /* until EOF */

  if (fcntl(fd, F_SETLK, &lock) < 0)
  { 
    if (errno == EACCES || errno == EAGAIN)
    {
      DEBUGMSGTL(("vipnet", "file fd %d is locked!\n", fd));
      return EAGAIN;
    }
    else
    {
      snmp_log(LOG_ERR,"can't set read lock for fd %d (errno %d).\n", fd, errno);
      DEBUGMSGTL(("vipnet", "can't set read lock for fd %d (errno %d)\n", fd, errno));
      return -errno;
    }
  }

  return 0;
}

void read_unlock(int fd)
{
	struct flock lock;
  lock.l_type = F_UNLCK;      /* UnLock file */
  lock.l_start = 0;           /* Offset where lock begins */
  lock.l_whence = SEEK_SET;   /* from current file-seek */
  lock.l_len = 0;             /* until EOF */

  fcntl(fd, F_SETLK, &lock);
}
