#include <linux/miscdevice.h>

/** The module name. */
#define IPLIR_DEV_NAME         "drviplir"
#define PROC_IPLIR             "driver/drviplir"
#define PROC_LOCAL_BUFFER_SIZE 32

#ifdef CONFIG_X86_64
#define DEFAULT_DRVIPLIR_PQSIZE 8192
#else
#define DEFAULT_DRVIPLIR_PQSIZE 2048
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
typedef unsigned long uintptr_t;
#endif

extern int GCC_NOREGPARM drviplir_open(VN_STINODE inode, VN_STFILE file);
extern int GCC_NOREGPARM drviplir_release(VN_STINODE inode, VN_STFILE file);
extern int GCC_NOREGPARM drviplir_ioctl( int cmd, uintptr_t arg );
extern int GCC_NOREGPARM drviplir_write(VN_STFILE file, char * buf,
                                        unsigned int count, void * fpos);
/*
** stubs to fill file_operations struct while charaster device register
** (see comments about CONFIG_REGPARM)
*/
int my_drviplir_open(struct inode *inode, struct file *file)
{
    return drviplir_open( (VN_STINODE)inode, (VN_STFILE)file);
}

int my_drviplir_release( struct inode *inode, struct file * file)
{
    return drviplir_release( (VN_STINODE)inode, (VN_STFILE)file);
}

//#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
#ifdef HAVE_UNLOCKED_IOCTL
long my_drviplir_ioctl( struct file *file, unsigned int cmd, uintptr_t arg)
{
    return drviplir_ioctl(cmd, arg);
}
#else
int my_drviplir_ioctl( struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    return drviplir_ioctl(cmd, arg);
}
#endif

int my_drviplir_write(struct file * file, char * buf, unsigned int count, void * pos)
{
    return drviplir_write((VN_STFILE)file, buf, count, pos);
}

#ifdef CONFIG_ITCSDRV_AS_MISC
/** The file_operations structure. */
static struct file_operations gFileOpsIplir =
{
owner:      THIS_MODULE,
open:       my_drviplir_open,
release:    my_drviplir_release,
#ifdef HAVE_UNLOCKED_IOCTL
unlocked_ioctl: my_drviplir_ioctl,
#else
ioctl:      my_drviplir_ioctl,
#endif
#ifdef HAVE_COMPAT_IOCTL
compat_ioctl: my_drviplir_ioctl,
#endif
#ifdef USE_DEVICE_WRITE_COMMANDS
write:      my_drviplir_write
#endif
};

/** The miscdevice structure. */
static struct miscdevice g_IplirDev =
{
minor:      MISC_DYNAMIC_MINOR,
name:       IPLIR_DEV_NAME,
fops:       &gFileOpsIplir,
# if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17)
devfs_name: IPLIR_DEV_NAME
# endif
};
#else //!CONFIG_ITCSDRV_AS_MISC
static int dev_major_number = 0;
#endif

int drviplir_pqsize_decrease = 8;
module_param( drviplir_pqsize_decrease , int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC( drviplir_pqsize_decrease , "maximum number of processed packets for releasing queue's space");

int drviplir_pqsize_overflow_grace  = 3; /* 87.5% of drviplir_pqsize*/
module_param( drviplir_pqsize_overflow_grace  , int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC( drviplir_pqsize_overflow_grace  , "packet queue end of overflow grace, use as factor: 0 - empty queue, 1 - 50\%, 2 -75\%, 3 - 87.5\% of drviplir_pqsize");

int drviplir_batch_size = 4;
module_param( drviplir_batch_size, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC( drviplir_batch_size, "maximum number of packets those are being processed in one iteration");

/* Reserve space for 1024 ARP packets + pqsize of local traffic packets */
int drviplir_pqsize_urgent = 1024 + DEFAULT_DRVIPLIR_PQSIZE;
module_param( drviplir_pqsize_urgent , int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC( drviplir_pqsize_urgent , "reserve space in packet queue for \"urgent\" packets");

int drviplir_relax_count = 32768; /*in HW2000 we release CPU0 for interrupts processing and other common task, so we need relax cpu rarely, only for core system functionality like workqueues and RCU  */
module_param( drviplir_relax_count, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC( drviplir_relax_count, "Release cpu after handling that amount of packets");

int drviplir_pqsize = DEFAULT_DRVIPLIR_PQSIZE;
module_param(drviplir_pqsize, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC(drviplir_pqsize, "Size of the packet queue");

static int drviplir_major = -1;
module_param(drviplir_major, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC(drviplir_major, "Major number for iplir driver");

int threads_count = 0;
int threads_maximum = 0;
module_param(threads_count, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC(threads_count, "The number of threads driver");

int threads_mask = 0xff;
int cpu_use_mask = 0;
module_param( cpu_use_mask, int, S_IRUGO );
MODULE_PARM_DESC(cpu_use_mask, "Cpu use mask");

#ifdef  DRV_TRACE_RUNTIME
int trace_mask = 0;
module_param(trace_mask, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
MODULE_PARM_DESC(trace_mask, "The packet in-out trace mack");
#endif

#if  LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32)
#define CALC_CHECKSUM_BY_HARDWARE 0
#else
#define CALC_CHECKSUM_BY_HARDWARE 1
#endif
int hw_csum = CALC_CHECKSUM_BY_HARDWARE;
module_param(hw_csum, int, S_IRUSR | S_IRUSR | S_IRGRP | S_IRGRP);
MODULE_PARM_DESC(hw_csum, "Hardware/software transport header checksum calculation.");

/*
** 2.6 kernel requires module_init() and module_exit() macros,
** so it's now redone like this. --vel
*/
extern int drviplir_real_init_module(void);
extern void drviplir_real_cleanup_module(void);

static int check_enable_usefull_symbols(void)
{
#if defined(CONFIG_KALLSYMS)
     /* Find adresses of ip_rcv_finish & ip_finish_output */
     myip_rcv_finish = (int (*)(struct sk_buff*))mykallsyms_lookup_name("ip_rcv_finish");
     myip_finish_output = (int (*)(struct sk_buff*))mykallsyms_lookup_name("ip_finish_output");
     if( myip_rcv_finish == 0 )
     {
          myprintk("Cannot find ip_rcv_finish in your kernel!\n");
     }
     if( myip_finish_output == 0 )
     {
          myprintk("Cannot find ip_finish_output in your kernel!\n");
     }
#endif

     return 0;
}

static struct proc_dir_entry *iplir_proc_dir /*= 0*/ ;


static int threads_count_proc_show(struct seq_file *m, void *v)
{
     int ret;

     ret=seq_printf( m, "%d\n", threads_count);
     return ret;
}

#ifdef  DRV_TRACE_RUNTIME
static int trace_mask_proc_show(struct seq_file *m, void *v)
{
     int ret;

     ret=seq_printf( m, "%d\n", trace_mask);
     return ret;
}
#endif

#ifdef  USE_FRWL_STATS
static int frwl_counters_proc_show(struct seq_file *m, void *v)
{
     int ret;

     ret = seq_printf( m, "\n Statistics: \n" );
     myfrwl_stats_proc_show( m );

     ret = seq_printf( m, "\n Errors: \n" );
     myfrwl_errors_proc_show( m );

     return ret;
}
#endif

static int threads_count_proc_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
     return single_open(file, threads_count_proc_show,PDE(inode)->data);
#else
     return single_open(file, threads_count_proc_show, PDE_DATA(inode));
#endif
}

#ifdef  DRV_TRACE_RUNTIME
static int trace_mask_proc_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
     return single_open(file, trace_mask_proc_show,PDE(inode)->data);
#else
     return single_open(file, trace_mask_proc_show, PDE_DATA(inode));
#endif
}
#endif

#ifdef  USE_FRWL_STATS
static int frwl_counters_proc_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
     return single_open(file, frwl_counters_proc_show,PDE(inode)->data);
#else
     return single_open(file, frwl_counters_proc_show, PDE_DATA(inode));
#endif
}
#endif

extern int GCC_NOREGPARM PacketQueueChange( const int cpu_count );

static int threads_count_proc_write(struct file *file, const char /*__user*/ *buf, size_t count, loff_t *pos)
{
     char str[PROC_LOCAL_BUFFER_SIZE];
     size_t len;
     int value;
     int rc;

     len = min(count, sizeof(str) - 1);
     if (copy_from_user(str, buf, len))
          return -EFAULT;
     str[len] = '\0';

     if (sscanf(str, "%i", &value) == 1 &&
         value >= 1 && value <= threads_maximum)
     {
          threads_count = value;
          rc = PacketQueueChange( value ); 
          if ( rc)
          {
               myprintk("error %d change threads count\n", rc);
               return -EIO;
          }
     } else
     {
          myprintk("error change threads count\n");
          return -EINVAL;
     }

     return count;
}

#ifdef  DRV_TRACE_RUNTIME
static int trace_mask_proc_write(struct file *file, const char /*__user*/ *buf, size_t count, loff_t *pos)
{
     char str[PROC_LOCAL_BUFFER_SIZE];
     size_t len;
     int value;
     int rc;

     len = min(count, sizeof(str) - 1);
     if (copy_from_user(str, buf, len))
          return -EFAULT;
     str[len] = '\0';

     if (sscanf(str, "%i", &value) == 1 )
     {
          trace_mask = value;
     } 
     else
     {
          myprintk("error change trace mask\n");
          return -EINVAL;
     }

     return count;
}
#endif

#ifdef  USE_FRWL_STATS
static int frwl_counters_proc_write(struct file *file, const char /*__user*/ *buf, size_t count, loff_t *pos)
{
     char str[PROC_LOCAL_BUFFER_SIZE];
     size_t len;
     int value;
     char value2;
     int rc;

     len = min(count, sizeof(str) - 1);
     if (copy_from_user(str, buf, len))
          return -EFAULT;
     str[len] = '\0';

     if (sscanf(str, "%i", &value) == 1 )
     {
          if (value >=0 && value <= 1)
               myfrwl_toggle_counters( value );
     }
     else if (sscanf(str, "%c", &value2) == 1 &&
    		 (value2 == 'f' || value2 == 'F') )
     {
          myfrwl_flush_counters();
     }
     else
     {
          myprintk("error arg\n");
          return -EINVAL;
     }

     return count;
}
#endif

static const struct file_operations threads_count_proc_fops = {
     .owner	= THIS_MODULE,
     .open	= threads_count_proc_open,
     .read	= seq_read,
     .llseek	= seq_lseek,
     .release	= single_release,
     .write	= threads_count_proc_write,
};

#ifdef  DRV_TRACE_RUNTIME
static const struct file_operations trace_mask_proc_fops = {
     .owner = THIS_MODULE,
     .open  = trace_mask_proc_open,
     .read  = seq_read,
     .llseek    = seq_lseek,
     .release   = single_release,
     .write = trace_mask_proc_write,
};
#endif

#ifdef USE_FRWL_STATS
static const struct file_operations frwl_counters_proc_fops = {
     .owner = THIS_MODULE,
     .open  = frwl_counters_proc_open,
     .read  = seq_read,
     .llseek    = seq_lseek,
     .release   = single_release,
     .write = frwl_counters_proc_write,
};
#endif

int drviplir_proc_create(void)
{
     struct proc_dir_entry *pde;

     iplir_proc_dir = proc_mkdir(PROC_IPLIR, NULL);
     if( iplir_proc_dir == NULL )
     {
          myprintk("Unable to create proc dir " PROC_IPLIR "\n");
          return -ENODEV;
     }
     
// thread_count
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = proc_create_data("thread_count", 0 /* default mode */,
                      iplir_proc_dir, &threads_count_proc_fops,
                      NULL /* client data */);
#else //LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = create_proc_entry("thread_count", 0 /* default mode */,
                      iplir_proc_dir);
     if (pde)
     {
          pde->proc_fops = &threads_count_proc_fops;
     }
#endif // LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     if (!pde)
     {
          myprintk("Failed to create proc entry thread_count\n");
          return -1;
     }

#ifdef  DRV_TRACE_RUNTIME     
// trace_mask
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = proc_create_data("trace_mask", 0 /* default mode */,
                      iplir_proc_dir, &trace_mask_proc_fops,
                      NULL /* client data */);
#else //LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = create_proc_entry("trace_mask", 0 /* default mode */,
                      iplir_proc_dir);
     if (pde)
     {
          pde->proc_fops = &trace_mask_proc_fops;
     }
     
#endif // LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     if (!pde)
     {
          myprintk("Failed to create proc entry trace_mask\n");
          return -1;
     }
#endif

#ifdef  USE_FRWL_STATS
// frwl_counters
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = proc_create_data("frwl_counters", 0 /* default mode */,
                      iplir_proc_dir, &frwl_counters_proc_fops,
                      NULL /* client data */);
#else //LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     pde = create_proc_entry("frwl_counters", 0 /* default mode */,
                      iplir_proc_dir);
     if (pde)
     {
          pde->proc_fops = &frwl_counters_proc_fops;
     }

#endif // LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
     if (!pde)
     {
          myprintk("Failed to create proc entry frwl_counters\n");
          return -1;
     }
#endif
     
     return 0;
}

int drviplir_proc_remove(void)
{
     if (iplir_proc_dir)
     {
#ifdef  DRV_TRACE_RUNTIME       
          remove_proc_entry("trace_mask", iplir_proc_dir);
#endif

#ifdef USE_FRWL_STATS
          remove_proc_entry("frwl_counters", iplir_proc_dir);
#endif
          remove_proc_entry("thread_count", iplir_proc_dir);
          remove_proc_entry(PROC_IPLIR, NULL);
     }

     return 0;
}

static int __init drviplir_init_module(void)
{
     int i,rc=0;
     int useCpuCount;

    check_enable_usefull_symbols();

     // setup threads count
#ifdef MAX_IPLIR_THREADS
     threads_maximum = MAX_IPLIR_THREADS;
#else
     threads_maximum = mynum_online_cpus() * 4;
#endif // MAX_IPLIR_THREADS
     threads_maximum = threads_maximum < 32 ? threads_maximum : 32;

     useCpuCount = mynum_online_cpus() < threads_maximum ? mynum_online_cpus() : threads_maximum;

     udebug(("online cpu's %u.\n", mynum_online_cpus() ));
     udebug(("use cpu's %u.\n", useCpuCount ));
     // threads count may be initialized in iplir.c as module params
     if( threads_count <= 0 )
     {
          threads_count = useCpuCount;
     }
     if ( threads_count > threads_maximum )
     {
          threads_count = threads_maximum;
     }
     myprintk("iplir threads count %d.\n", threads_count);

     threads_mask = 0;
     if( cpu_use_mask == 0 )
     {
          for( i = 0; i < useCpuCount; i++ )
          {
               threads_mask |= ( 1 << i ); 
          }
     }
     else
     {
          for( i = 0; i < useCpuCount; i++ )
          {
               if( cpu_use_mask & ( 1 << i ) )
                    threads_mask |= ( 1 << i );
          }
          myprintk("threads mask: 0x%x\n", threads_mask);
     }

     if ( (drviplir_pqsize < 0) || (drviplir_pqsize > 16384 ) )
     {
          drviplir_pqsize = DEFAULT_DRVIPLIR_PQSIZE;
     }
     myprintk("size of packet queue is %d\n", drviplir_pqsize);

#ifdef CONFIG_ITCSDRV_AS_MISC
     rc = misc_register(&g_IplirDev);
     if (rc)
     {
          printk(KERN_ERR " Can't register " IPLIR_DEV_NAME " device! rc=%d\n", rc);
          return rc;
     }
#else  /* !CONFIG_ITCSDRV_AS_MISC */
     /*
     * Register character device.
     */

     if (drviplir_major != -1)
     {
          if ((drviplir_major <= 0) || (drviplir_major > 254))
          {
               myprintk("IplirDrv: device major number is incorrectly specified (%d)\n",
                        drviplir_major);
               return myENODEV();
          }
          dev_major_number = drviplir_major;
     }

     rc = register_chrdev((dev_t)dev_major_number, IPLIR_DEV_NAME, &gFileOpsIplir);
     if (rc < 0)
     {
          myprintk("IplirDrv: cannot register device (code %d)\n", rc);
          return myENODEV();
     }

    /*
     * Save returned module major number
     */
     if ( dev_major_number == 0 )
          dev_major_number = rc;
     rc = 0;

# ifdef CONFIG_DEVFS_FS
    /*
     * Register a device entry
     */
     if (devfs_mk_cdev(MKDEV(dev_major_number, 0), S_IFCHR | ITCS_DEV_FMASK, IPLIR_DEV_NAME) != 0)
     {
          myprintk("devfs_register failed!\n");
          return myENODEV();
     }
# endif
     myprintk("IplirDrv: device number = %d, device node = /dev/"
              IPLIR_DEV_NAME "\n", dev_major_number);

#endif /* !CONFIG_ITCSDRV_AS_MISC */

     rc = drviplir_real_init_module();
     if  ( rc)
     {
        /** Failed, cleanup and return the error code.
         */
          myprintk("drviplir_real_init_module returning %#x)\n", rc);
#if defined(CONFIG_DEVFS_FS) && !defined(CONFIG_ITCSDRV_AS_MISC)
          devfs_remove(IPLIR_DEV_NAME);
#endif

#ifdef CONFIG_ITCSDRV_AS_MISC
          misc_deregister(&g_IplirDev);
#else
          unregister_chrdev(dev_major_number, IPLIR_DEV_NAME);
#endif
          return rc;
    }

     rc = drviplir_proc_create();
     if  ( rc)
     {
        /** Failed, cleanup and return the error code.
         */
          myprintk("drviplir_proc_create returning %#x)\n", rc);
#if defined(CONFIG_DEVFS_FS) && !defined(CONFIG_ITCSDRV_AS_MISC)
          devfs_remove(IPLIR_DEV_NAME);
#endif

#ifdef CONFIG_ITCSDRV_AS_MISC
          misc_deregister(&g_IplirDev);
#else
          unregister_chrdev(dev_major_number, IPLIR_DEV_NAME);
#endif
          return rc;
     }

     myprintk("ViPNet IPLIR Driver successfully loaded\n");

     return rc;
}

static void __exit drviplir_cleanup_module(void)
{
     int                 rc;

     drviplir_proc_remove();

     drviplir_real_cleanup_module();

     myprintk("Unload\n");
     NOREF(rc);

/** I Don't think it's possible to unload a driver which processes have
 * opened, at least we'll blindly assume that here.
 */
#ifdef CONFIG_ITCSDRV_AS_MISC
     rc = misc_deregister(&g_IplirDev);
     if (rc < 0)
     {
          myprintk("misc_deregister failed with rc=%#x\n", rc);
     }
#else  /* !CONFIG_ITCSDRV_AS_MISC */
# ifdef CONFIG_DEVFS_FS
    /*
     * Unregister a device entry
     */
     devfs_remove(IPLIR_DEV_NAME);
# endif /* devfs */
     unregister_chrdev(dev_major_number, IPLIR_DEV_NAME );
#endif /* !CONFIG_ITCSDRV_AS_MISC */

}

module_init(drviplir_init_module);
module_exit(drviplir_cleanup_module);

extern char* get_crypto_handle( void );
