/* itcskrniface. Interface to Linux kernel functions. */

#include "itcskrniface.h"

#include <net/route.h> /* for ip_route_output_key */
#include <net/ipv6.h> /* for __ip6_local_out */

#ifdef CONFIG_X86
#include <asm/i387.h>
#endif

unsigned long pf_kallsyms_lookup_name = 0;
module_param_named(lookup_address,pf_kallsyms_lookup_name, ulong, S_IRUSR | S_IRGRP);
MODULE_PARM_DESC(pf_kallsyms_lookup_name, "kallsyms_lookup_name() address");
#ifdef CONFIG_ITCSDRV_AS_MISC
/** The miscdevice structure. */
static struct miscdevice gItcsDevice =
{
minor:      MISC_DYNAMIC_MINOR,
name:       DEVICE_NAME,
fops:       NULL,
# if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17)
devfs_name: DEVICE_NAME,
# endif
};
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
extern int nf_register_afinfo(const struct nf_afinfo *afinfo);
extern void nf_unregister_afinfo(const struct nf_afinfo *afinfo);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
extern int nf_register_afinfo(struct nf_afinfo *afinfo);
extern void nf_unregister_afinfo(struct nf_afinfo *afinfo);
#endif
/**
 * mysched_setscheduler - acording to the linux version calls
 * respective setscheduler function to change the scheduling
 * policy and/or RT priority of a thread.
 * @p: the task in question.
 * @policy: new policy.
 * @param: structure containing the new RT priority.
 *
 * NOTE that the task may be already dead.
 */
int mysched_setscheduler(struct task_struct *p, int policy,
		       struct sched_param *param)
{
    return sched_setscheduler( p, policy, param );
}
EXPORT_SYMBOL(mysched_setscheduler);

void myemergency_restart(void)
{
    emergency_restart();
}

EXPORT_SYMBOL(myemergency_restart);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
int mynf_register_afinfo(const struct nf_afinfo *afinfo)
{
        return nf_register_afinfo(afinfo);
}
EXPORT_SYMBOL(mynf_register_afinfo);

void mynf_unregister_afinfo(const struct nf_afinfo *afinfo)
{
        nf_unregister_afinfo(afinfo);
}
EXPORT_SYMBOL(mynf_unregister_afinfo);
#endif


int myip_route_output_key(struct rtable ** rt, unsigned int dest_ip4, unsigned int *src_ip4)
{
#if  LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39)
     int errStatus;
     struct flowi fl;
     memset(&fl, 0, sizeof(fl));
     fl.nl_u.ip4_u.daddr = dest_ip4;
#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
     struct flowi fl;
     memset(&fl, 0, sizeof(fl));
     fl.u.ip4.daddr = dest_ip4;
#else
     struct flowi4 fl;
     memset(&fl, 0, sizeof(fl));
     fl.daddr = dest_ip4;
#endif


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,25)
     errStatus = ip_route_output_key(rt, &fl);
     if( errStatus )
     {
          return errStatus;
     }
     *src_ip4=(*rt)->rt_src;
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39)
     errStatus = ip_route_output_key(&init_net, rt, &fl);
     if( errStatus )
     {
          return errStatus;
     }
     *src_ip4=(*rt)->rt_src;
#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
     /* since 2.6.39 ip_route_output_key have changed interface */
     if (IS_ERR(*rt = ip_route_output_key(&init_net, &fl.u.ip4)))
     {
          return PTR_ERR(*rt);
     }
     *src_ip4=(*rt)->rt_src;
#else
     if (IS_ERR(*rt = ip_route_output_key(&init_net, &fl)))
     {
          return PTR_ERR(*rt);
     }
     *src_ip4=fl.saddr;
#endif
     return 0;
}
EXPORT_SYMBOL(myip_route_output_key);

int myip_check_route_output_key(struct rtable ** rt, unsigned int dest_ip4)
{
     unsigned int dummy_src_ip;
     return myip_route_output_key(rt, dest_ip4, &dummy_src_ip);
}
EXPORT_SYMBOL(myip_check_route_output_key);


struct task_struct *myfind_task_by_pid(pid_t pid)
{
    struct task_struct * tsk;
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 24)
    rcu_read_lock();
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
    tsk = pid_task(find_pid_ns(pid, current->nsproxy->pid_ns_for_children), PIDTYPE_PID);
#else //LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
    tsk = pid_task(find_pid_ns(pid, current->nsproxy->pid_ns), PIDTYPE_PID);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(3, 11, 0)
    if(tsk)
        myget_task_struct((struct task_struct *)tsk);
    rcu_read_unlock();
#else
#ifdef find_task_by_pid_all
// In case if kernel supports OpenVZ
    tsk = find_task_by_pid_all(pid);
#else
    tsk = find_task_by_pid(pid);
#endif //find_task_by_pid_all
#endif //LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 24)
	return tsk;
}
EXPORT_SYMBOL(myfind_task_by_pid);

void myput_task_struct_lite(struct task_struct *t)
{
#if LINUX_VERSION_CODE >  KERNEL_VERSION(2, 6, 24)
     if(t)
	 {
	      (void) atomic_dec_and_test(&t->usage);
     }
#endif
}
EXPORT_SYMBOL(myput_task_struct_lite);

int my__set_affinity(int cpu)
{
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 25)
     return set_cpus_allowed(current, cpumask_of_cpu(cpu));
#else
     return set_cpus_allowed_ptr(current, &cpumask_of_cpu(cpu));
#endif
}
EXPORT_SYMBOL(my__set_affinity);

unsigned long mykallsyms_lookup_name(const char *name)
{
#if defined(CONFIG_KALLSYMS) && KALLSYMS_LOOKUP_NAME_EXPORT
     return kallsyms_lookup_name(name);
#else
     typedef unsigned long (*PF)(const char*);
     if( pf_kallsyms_lookup_name )
     {
    	 return ( (PF) pf_kallsyms_lookup_name )(name);
     }
     return 0;
#endif
}
EXPORT_SYMBOL(mykallsyms_lookup_name);

void* itcs_symbol_get( const char *symbol)
{
     return __symbol_get( symbol );
}
EXPORT_SYMBOL(itcs_symbol_get);

void itcs_symbol_put( const char *symbol)
{
     __symbol_put( symbol );
}
EXPORT_SYMBOL(itcs_symbol_put);

#if (defined(__i386__) || defined(__x86_64__))
void my_kernel_fpu_begin( void )
{
     kernel_fpu_begin();
}
EXPORT_SYMBOL(my_kernel_fpu_begin);

void my_kernel_fpu_end( void )
{
     kernel_fpu_end();
}
EXPORT_SYMBOL(my_kernel_fpu_end);
#else 

/*
 * only X86 platform have kernel_fpu_begin/kernel_fpu_end
 * see tfs:108225 + notes for details before change
 */
#define my_kernel_fpu_begin
#define my_kernel_fpu_end

#endif 

/**
 * init_module:
 *
 * When the driver is loaded as a module this function is called.
 *
 * Returns 0 for success or anything else on error. Returning an error
 * here also causes the module to be unloaded
 */
int __init init_module(void)
{
    int       rc;

#ifdef CONFIG_ITCSDRV_AS_MISC
    rc = misc_register(&gItcsDevice);
    if (rc)
    {
        printk(KERN_ERR DEVICE_NAME ": Can't register misc device! rc=%d\n", rc);
        return rc;
    }
#else  /* !CONFIG_ITCSDRV_AS_MISC */
/*
     * Register character device.
     */
    g_iModuleMajor = DEVICE_MAJOR;
    rc = register_chrdev((dev_t)g_iModuleMajor, DEVICE_NAME,NULL);
    if (rc < 0)
    {
        printk(KERN_ALERT "itcskrniface: register_chrdev() failed with rc=%#x!\n", rc);
        return rc;
    }

    /*
     * Save returned module major number
     */
    if (DEVICE_MAJOR != 0)
        g_iModuleMajor = DEVICE_MAJOR;
    else
        g_iModuleMajor = rc;
    rc = 0;

# ifdef CONFIG_DEVFS_FS
    /*
     * Register a device entry
     */
    if (devfs_mk_cdev(MKDEV(DEVICE_MAJOR, 0), S_IFCHR | ITCS_DEV_FMASK, DEVICE_NAME) != 0)
    {
        printk( KERN_ALERT "itcskrniface:devfs_register failed!\n");
        rc = -EINVAL;
    }
 * When the driver is loaded as a module this function is called. 
# endif
#endif /* !CONFIG_ITCSDRV_AS_MISC */

	printk(KERN_ALERT "itcskrniface: Kernel interface driver is loaded\n");
	return 0;
}

/**
 * cleanup_module:
 *
 * The module is being unloaded. We free up the resources we used.
 */

void __exit cleanup_module(void)
{
    int                 rc;
    NOREF(rc);

#ifdef  CONFIG_ITCSDRV_AS_MISC
    rc = misc_deregister(&gItcsDevice);
    if (rc < 0)
    {
        printk(KERN_ALERT "misc_deregister failed with rc=%#x\n", rc);
    }
#else  /* !CONFIG_ITCSDRV_AS_MISC */
# ifdef CONFIG_DEVFS_FS
    /*
     * Unregister a device entry
     */
    devfs_remove(DEVICE_NAME);
# endif /* CONFIG_DEVFS_FS */
    unregister_chrdev(g_iModuleMajor, DEVICE_NAME);
#endif /* !CONFIG_ITCSDRV_AS_MISC */

    printk(KERN_ALERT "itcskrniface: Kernel interface driver is unloaded\n");
}


void myrcu_read_lock(void)
{
     rcu_read_lock();
}
EXPORT_SYMBOL(myrcu_read_lock);


void myrcu_read_unlock(void)
{
     rcu_read_unlock();
}
EXPORT_SYMBOL(myrcu_read_unlock);


void myrcu_read_lock_bh(void)
{
     rcu_read_lock_bh();
}
EXPORT_SYMBOL(myrcu_read_lock_bh);


void myrcu_read_unlock_bh(void)
{
     rcu_read_unlock_bh();
}
EXPORT_SYMBOL(myrcu_read_unlock_bh);


struct in_device *myin_dev_get(const struct net_device *dev)
{
     return in_dev_get(dev);
}
EXPORT_SYMBOL(myin_dev_get);

void* mymemmove(void* dest, const void* src, size_t count)
{
     return memmove( dest, src, count );
}
EXPORT_SYMBOL(mymemmove);

void* mymemcmp(const void* dest, const void* src, size_t count)
{
	return (void*) memcmp( dest, src, count );
}
EXPORT_SYMBOL(mymemcmp);

void* mymemcpy(void* dest, const void* src, size_t count)
{
	return (void*) memcpy( dest, src, count );
}
EXPORT_SYMBOL(mymemcpy);

int myskb_cow_data_krnface(struct sk_buff *skb_out, int tailbits, struct sk_buff **trailer_out)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	return skb_cow_data(skb_out, tailbits, trailer_out);
#else
	return NULL;
#endif
}
EXPORT_SYMBOL(myskb_cow_data_krnface);

int my_ip6_local_out_krnface(struct sk_buff *skb_out)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25) && defined(CONFIG_IPV6)
	return ip6_local_out(skb_out);
#else
	return 0;
#endif
}
EXPORT_SYMBOL(my_ip6_local_out_krnface);

int mycancel_work_sync_krnface(struct work_struct *work)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
  return cancel_work_sync(work);
#else
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
  cancel_work_sync(work);
#else
  (void)work;
  flush_scheduled_work();
#endif
  return 0;
#endif
}
EXPORT_SYMBOL(mycancel_work_sync_krnface);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)

void __percpu * __myalloc_percpu(size_t size, size_t align)
{
	return __alloc_percpu(size, align);
}
EXPORT_SYMBOL(__myalloc_percpu);

void myfree_percpu(void __percpu *ptr)
{
	free_percpu(ptr);
}
EXPORT_SYMBOL(myfree_percpu);

#endif

MODULE_AUTHOR("Infotecs");
MODULE_DESCRIPTION("Interface to Linux kernel functions");
MODULE_LICENSE("GPL");

