#ifndef __ITCSKRNIFACE_H__
#define __ITCSKRNIFACE_H__


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/version.h>
#include <linux/netfilter.h>
#include <asm/unistd.h>
#include <linux/interrupt.h>
#include <linux/cpumask.h>
#include <linux/kallsyms.h>
#include <linux/reboot.h>
#include <linux/netdevice.h>
#include <linux/inetdevice.h>
#include <linux/skbuff.h>
#include <linux/workqueue.h>
#include <net/route.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
#include <linux/nsproxy.h>
#endif

#ifdef CONFIG_DEVFS_FS
# include <linux/devfs_fs_kernel.h>
#endif

/** @def NOREF
 * Keeps the compiler from bitching about an unused parameter.
 */
#define NOREF(var)               (void)(var)
#define CONFIG_ITCSDRV_AS_MISC

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)

void __percpu * __myalloc_percpu(size_t size, size_t align);
void myfree_percpu(void __percpu *ptr);

#define myalloc_percpu(type)  \
          (typeof(type) __percpu *)__myalloc_percpu(sizeof(type), __alignof__(type))
#endif

#ifdef CONFIG_ITCSDRV_AS_MISC
# include <linux/miscdevice.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
#  if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#    define FL_FLOWI(fl)     struct flowi fl
#    define FL_SADDR(fl)     fl.src
#    define FL_DADDR(fl)     fl.dst
#  elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39)
#    define FL_FLOWI(fl)     struct flowi fl
#    define FL_SADDR(fl)     fl.nl_u.ip4_u.saddr
#    define FL_DADDR(fl)     fl.nl_u.ip4_u.daddr
#  else
#    define FL_FLOWI(fl)     struct flowi fl
#    define FL_SADDR(fl)     fl.u.ip4.saddr
#    define FL_DADDR(fl)     fl.u.ip4.daddr
#  endif
#else
#  define FL_FLOWI(fl)     struct flowi4 fl
#  define FL_SADDR(fl)     fl.saddr
#  define FL_DADDR(fl)     fl.daddr
#endif

#ifdef CONFIG_DEVFS_FS
# include <linux/devfs_fs_kernel.h>
#endif

#ifndef GCC_NOREGPARM
#if defined(LINUX_VERSION_CODE)
#if defined(__arm__)
      // modern/risc arch. always use registers in function calls
#     define GCC_NOREGPARM
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)) || \
     ((LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,19)) && (defined(CONFIG_REGPARM) && ( __GNUC__ >= 3 )))
#     define GCC_NOREGPARM __attribute__((regparm(0)))
#     define KERNEL_USE_REGPARM
#else
#     define GCC_NOREGPARM
#endif
#else  // !defined LINUX_VERSION_CODE
#     define GCC_NOREGPARM
#endif
#endif // define GCC_NOREGPARM


/** @def NOREF
 * Keeps the compiler from bitching about an unused parameter.
 */
#define NOREF(var)               (void)(var)
#define CONFIG_ITCSDRV_AS_MISC


#ifdef CONFIG_ITCSDRV_AS_MISC
# include <linux/miscdevice.h>
#endif

#ifndef CONFIG_ITCSDRV_AS_MISC
/** Module major number */
#define DEVICE_MAJOR        254
/** Saved major device number */
static int      g_iModuleMajor;
#endif /* !CONFIG_VBOXDRV_AS_MISC */

/** The module name. */
#define DEVICE_NAME         "itcskrniface"

/* devfs defines */
#if defined(CONFIG_DEVFS_FS) && !defined(CONFIG_ITCSDRV_AS_MISC)
#  define ITCS_DEV_FMASK     (S_IRUGO | S_IWUGO)
#endif /* CONFIG_DEV_FS && !CONFIG_VBOXDEV_AS_MISC */

int mysched_setscheduler(struct task_struct *p, int policy,
                       struct sched_param *param);
void myemergency_restart(void);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
int mynf_register_afinfo(const struct nf_afinfo *afinfo);
void mynf_unregister_afinfo(const struct nf_afinfo *afinfo);
#endif


int myip_route_output_key(struct rtable ** rt, unsigned int dest_ip4, unsigned int * src_ip4);

//this function is used when we want to check if route exists, but we 
//don't want to use it's address
int myip_check_route_output_key(struct rtable ** rt, unsigned int dest_ip4);

struct task_struct *myfind_task_by_pid(pid_t vnr);
#define myget_task_struct(tsk) get_task_struct(tsk)
void myput_task_struct_lite(struct task_struct *t);
int my__set_affinity(int cpu);

unsigned long mykallsyms_lookup_name(const char *name);
void my_kernel_fpu_begin( void );
void my_kernel_fpu_end( void );
#define KALLSYMS_LOOKUP_NAME_EXPORT LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)

void ** myget_sct(void);
void myrcu_read_lock(void);
void myrcu_read_unlock(void);
void myrcu_read_lock_bh(void);
void myrcu_read_unlock_bh(void);
struct in_device *myin_dev_get(const struct net_device *dev); 
void* mymemmove(void* dest, const void* src, size_t count);
void* mymemcmp(const void* dest, const void* src, size_t count);
void* mymemcpy(void* dest, const void* src, size_t count);
void* itcs_symbol_get( const char *symbol);
void itcs_symbol_put( const char *symbol);

int myskb_cow_data_krnface(struct sk_buff *skb_out, int tailbits, struct sk_buff **trailer_out);
int my_ip6_local_out_krnface(struct sk_buff *skb_out);
int mycancel_work_sync_krnface(struct work_struct *work);

#endif /* __ITCSKRNIFACE_H__ */
